#include <config.h>
#ifndef lint
static const char yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93";
#endif

#include <stdlib.h>
#include <string.h>

#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define YYPATCH 20070509

#define YYEMPTY (-1)
#define yyclearin    (yychar = YYEMPTY)
#define yyerrok      (yyerrflag = 0)
#define YYRECOVERING (yyerrflag != 0)

extern int yyparse(void);

static int yygrowstack(void);
#define YYPREFIX "yy"
#line 32 "p.y"
  
/*
 * DESCRIPTION
 *   Simple context-free grammar for parsing the control file. 
 *
 *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
 *  @author Olivier Beyssac, <ob@r14.freenix.org>
 *  @author Kianusch Sayah Karadji <kianusch.sayah.karadji@sk-tech.net>
 *  @author Martin Pala <martinp@tildeslash.com>
 *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
 *  @author Rory Toma <rory@digeo.com>
 */

#include <config.h>

#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif 

#ifdef HAVE_CTYPE_H
#include <ctype.h>
#endif 

#ifdef HAVE_PWD_H
#include <pwd.h>
#endif 

#ifdef HAVE_GRP_H
#include <grp.h>
#endif 
  
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
  
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
  
#ifdef HAVE_TIME_H
#include <time.h>
#endif
  
#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif

#ifdef HAVE_ASM_PARAM_H
#include <asm/param.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif

#ifdef HAVE_NETDB_H
#include <netdb.h>
#endif

#ifdef HAVE_SYSLOG_H
#include <syslog.h>
#endif

#ifndef HAVE_SOL_IP
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#endif

#ifdef HAVE_NETINET_IP_ICMP_H
#include <netinet/ip_icmp.h>
#endif

#ifdef HAVE_REGEX_H
#include <regex.h>
#endif
  
#include "net.h"
#include "monitor.h"
#include "protocol.h"
#include "engine.h"
#include "alert.h"
#include "process.h"
#include "ssl.h"
#include "device.h"


/* ------------------------------------------------------------- Definitions */

  struct IHavePrecedence {
    int daemon;
    int logfile;
    int pidfile;
  }; 

  struct myrate {
    unsigned count;
    unsigned cycles;
  }; 

  /* yacc interface */
  void  yyerror(const char *,...);
  void  yyerror2(const char *,...);
  void  yywarning(const char *,...);
  void  yywarning2(const char *,...);

  /* lexer interface */
  int yylex(void);
  extern FILE *yyin;
  extern int lineno;
  extern int arglineno;
  extern char *yytext;
  extern char *argyytext;
  extern char *currentfile;
  extern char *argcurrentfile;
  extern int buffer_stack_ptr;

  /* Local variables */
  static int cfg_errflag = FALSE;
  static Service_T tail = NULL;
  static Service_T current = NULL;
  static unsigned int eventset;
  static Request_T urlrequest = NULL;
  static Command_T command = NULL;
  static Command_T command1 = NULL;
  static Command_T command2 = NULL;
  static Service_T depend_list = NULL;
  static struct mygid gidset;
  static struct myuid uidset;
  static struct myperm permset;
  static struct mysize sizeset;
  static struct mymatch matchset;
  static struct myicmp icmpset;
  static struct mymail mailset;
  static struct myport portset;
  static struct mymailserver mailserverset;
  static struct myfilesystem filesystemset;
  static struct myresource resourceset;
  static struct mychecksum checksumset;
  static struct mytimestamp timestampset;
  static struct myactionrate actionrateset;
  static struct IHavePrecedence ihp = {FALSE, FALSE, FALSE};
  static struct myrate rate1 = {1, 1};
  static struct myrate rate2 = {1, 1};
  static char * htpasswd_file = NULL;
  static int    digesttype = DIGEST_CLEARTEXT;
  static int    hassystem = FALSE;

#define BITMAP_MAX (sizeof(long long) * 8)

  
/* -------------------------------------------------------------- Prototypes */

  static void  preparse();
  static void  postparse();
  static void  addservice(Service_T);
  static void  addmail(char *, Mail_T, Mail_T *, unsigned int, unsigned int);
  static void  createservice(int, char *, char *, int (*)(Service_T));
  static void  adddependant(char *);
  static void  addservicegroup(char *);
  static void  addport(Port_T);
  static void  addresource(Resource_T);
  static void  addtimestamp(Timestamp_T, int);
  static void  addactionrate(ActionRate_T);
  static void  addsize(Size_T);
  static void  addfilesystem(Filesystem_T);
  static void  addicmp(Icmp_T);
  static void *addprotocol(int);
  static void  addgeneric(Port_T, char*, char*);
  static void  addcommand(int, unsigned);
  static void  addargument(char *);
  static void  addmmonit(URL_T, int, int, char *);
  static void  addmailserver(MailServer_T);
  static int   addcredentials(char *, char *, int, int);
#ifdef HAVE_LIBPAM
  static void  addpamauth(char *, int);
#endif
  static void  addhtpasswdentry(char *, char *, int);
  static uid_t get_uid(char *, uid_t);
  static gid_t get_gid(char *, gid_t);
  static void  addchecksum(Checksum_T);
  static void  addperm(Perm_T);
  static void  addmatch(Match_T, int, int);
  static void  addmatchpath(Match_T, int);
  static void  adduid(Uid_T);
  static void  addgid(Gid_T);
  static void  addeuid(uid_t);
  static void  addegid(gid_t);
  static void  addeventaction(EventAction_T *, int, int);
  static void  seteventaction(EventAction_T *, int, int);
  static void  prepare_urlrequest(URL_T U);
  static void  seturlrequest(int, char *);
  static void  setlogfile(char *);
  static void  setpidfile(char *);
  static void  reset_mailset();
  static void  reset_mailserverset();
  static void  reset_portset();
  static void  reset_resourceset();
  static void  reset_timestampset();
  static void  reset_actionrateset();
  static void  reset_sizeset();
  static void  reset_checksumset();
  static void  reset_permset();
  static void  reset_uidset();
  static void  reset_gidset();
  static void  reset_filesystemset();
  static void  reset_icmpset();
  static void  reset_rateset();
  static void  check_name(char *);
  static void  check_every(int);
  static int   check_perm(int);
  static void  check_hostname (char *);
  static void  check_exec(char *);
  static int   cleanup_hash_string(char *);
  static void  check_depend();
  static void  setsyslog(char *);
  static void describeAction(Action_T);
  static Command_T copycommand(Command_T);
  static int verifyMaxForward(int);  

#line 266 "p.y"
typedef union {
  URL_T url;
  float real;
  int   number;
  char *string;
} YYSTYPE;
#line 263 "y.tab.c"
#define IF 257
#define ELSE 258
#define THEN 259
#define OR 260
#define FAILED 261
#define SET 262
#define LOGFILE 263
#define FACILITY 264
#define DAEMON 265
#define SYSLOG 266
#define MAILSERVER 267
#define HTTPD 268
#define ALLOW 269
#define ADDRESS 270
#define INIT 271
#define READONLY 272
#define CLEARTEXT 273
#define MD5HASH 274
#define SHA1HASH 275
#define CRYPT 276
#define DELAY 277
#define PEMFILE 278
#define ENABLE 279
#define DISABLE 280
#define HTTPDSSL 281
#define CLIENTPEMFILE 282
#define ALLOWSELFCERTIFICATION 283
#define IDFILE 284
#define STATEFILE 285
#define SEND 286
#define EXPECT 287
#define EXPECTBUFFER 288
#define CYCLE 289
#define COUNT 290
#define REMINDER 291
#define PIDFILE 292
#define START 293
#define STOP 294
#define PATHTOK 295
#define HOST 296
#define HOSTNAME 297
#define PORT 298
#define TYPE 299
#define UDP 300
#define TCP 301
#define TCPSSL 302
#define PROTOCOL 303
#define CONNECTION 304
#define ALERT 305
#define NOALERT 306
#define MAILFORMAT 307
#define UNIXSOCKET 308
#define SIGNATURE 309
#define TIMEOUT 310
#define RESTART 311
#define CHECKSUM 312
#define EVERY 313
#define DEFAULT 314
#define HTTP 315
#define APACHESTATUS 316
#define FTP 317
#define SMTP 318
#define POP 319
#define IMAP 320
#define CLAMAV 321
#define NNTP 322
#define NTP3 323
#define MYSQL 324
#define DNS 325
#define SSH 326
#define DWP 327
#define LDAP2 328
#define LDAP3 329
#define RDATE 330
#define RSYNC 331
#define TNS 332
#define PGSQL 333
#define POSTFIXPOLICY 334
#define SIP 335
#define LMTP 336
#define GPS 337
#define RADIUS 338
#define STRING 339
#define PATH 340
#define MAILADDR 341
#define MAILFROM 342
#define MAILSUBJECT 343
#define MAILBODY 344
#define SERVICENAME 345
#define STRINGNAME 346
#define NUMBER 347
#define PERCENT 348
#define LOGLIMIT 349
#define CLOSELIMIT 350
#define DNSLIMIT 351
#define KEEPALIVELIMIT 352
#define REPLYLIMIT 353
#define REQUESTLIMIT 354
#define STARTLIMIT 355
#define WAITLIMIT 356
#define GRACEFULLIMIT 357
#define CLEANUPLIMIT 358
#define REAL 359
#define CHECKPROC 360
#define CHECKFILESYS 361
#define CHECKFILE 362
#define CHECKDIR 363
#define CHECKHOST 364
#define CHECKSYSTEM 365
#define CHECKFIFO 366
#define CHECKSTATUS 367
#define CHILDREN 368
#define SYSTEM 369
#define RESOURCE 370
#define MEMORY 371
#define TOTALMEMORY 372
#define LOADAVG1 373
#define LOADAVG5 374
#define LOADAVG15 375
#define MODE 376
#define ACTIVE 377
#define PASSIVE 378
#define MANUAL 379
#define CPU 380
#define TOTALCPU 381
#define CPUUSER 382
#define CPUSYSTEM 383
#define CPUWAIT 384
#define GROUP 385
#define REQUEST 386
#define DEPENDS 387
#define BASEDIR 388
#define SLOT 389
#define EVENTQUEUE 390
#define SECRET 391
#define HOSTHEADER 392
#define UID 393
#define GID 394
#define MMONIT 395
#define INSTANCE 396
#define USERNAME 397
#define PASSWORD 398
#define TIMESTAMP 399
#define CHANGED 400
#define SECOND 401
#define MINUTE 402
#define HOUR 403
#define DAY 404
#define SSLAUTO 405
#define SSLV2 406
#define SSLV3 407
#define TLSV1 408
#define CERTMD5 409
#define BYTE 410
#define KILOBYTE 411
#define MEGABYTE 412
#define GIGABYTE 413
#define INODE 414
#define SPACE 415
#define PERMISSION 416
#define SIZE 417
#define MATCH 418
#define NOT 419
#define IGNORE 420
#define ACTION 421
#define EXEC 422
#define UNMONITOR 423
#define ICMP 424
#define ICMPECHO 425
#define NONEXIST 426
#define INVALID 427
#define DATA 428
#define RECOVERED 429
#define PASSED 430
#define SUCCEEDED 431
#define URL 432
#define CONTENT 433
#define PID 434
#define PPID 435
#define FSFLAG 436
#define URLOBJECT 437
#define TARGET 438
#define MAXFORWARD 439
#define GREATER 440
#define LESS 441
#define EQUAL 442
#define NOTEQUAL 443
#define YYERRCODE 256
short yylhs[] = {                                        -1,
    0,    0,    1,    1,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    2,   17,   17,   32,   32,   32,
   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,
   19,   19,   46,   46,   46,   46,   46,   46,   46,   46,
   46,   46,   46,   46,   46,   46,   46,   21,   21,   54,
   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,
   54,   54,   54,   23,   23,   58,   58,   58,   58,   58,
   58,   58,   58,   58,   58,   58,   58,   25,   25,   59,
   59,   59,   59,   59,   59,   59,   59,   59,   59,   27,
   27,   61,   61,   61,   61,   61,   61,   29,   29,   63,
   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,
   63,   31,   31,   64,   64,   64,   64,   64,    3,    3,
    3,    4,   69,   69,   14,   15,    5,    5,    5,    6,
    6,    6,   12,   13,   11,    7,   71,   71,   72,    8,
    9,   10,   76,   76,   80,   80,   79,   79,   83,   83,
   83,   83,   84,   84,   89,   89,   89,   88,   88,   88,
   88,   88,   85,   85,   93,   93,   94,   94,   86,   90,
   91,   91,   92,   92,   87,   87,   87,   87,   87,   87,
   97,   87,   98,   87,   99,   87,  100,   87,   87,   96,
   96,  101,   95,   95,   16,   16,   18,   20,   22,   24,
   26,   28,   30,   33,   33,   34,   34,  102,  102,  104,
  104,  105,  105,  106,  106,  106,  106,   81,   81,   81,
   82,   82,   77,   77,   37,   37,   38,   60,  107,  107,
  108,  115,  109,  109,  109,  109,   75,   75,   74,   74,
   74,   74,   74,  110,  110,  110,  110,  110,  110,  110,
  110,  110,  110,  110,  110,  110,  110,  110,  110,  110,
  110,  110,  110,  110,  110,  110,  110,  110,  110,  110,
  123,  123,  124,  124,  120,  120,  120,  121,  121,  119,
  119,  119,  125,  125,  122,  118,  118,  126,  126,  126,
  126,  126,  126,  126,  126,  126,  126,   35,   36,  117,
  117,  103,  103,   73,   73,   39,   39,  114,  114,  128,
  128,   40,   40,   40,   40,   65,  129,   66,   66,  130,
  130,  130,  130,  130,  130,  130,  130,  130,  130,  130,
  130,  130,  130,  130,  130,  130,  130,  130,  130,   67,
   67,   78,   78,  131,  131,  131,   41,   42,   42,   42,
   43,   44,  132,  132,  133,   45,  134,  134,  135,  135,
  135,  135,   62,  140,  140,  141,  141,  141,  136,  136,
  142,  143,  143,  143,  137,  137,  137,  137,  138,  139,
  145,  145,  145,  144,  144,   47,   47,  127,  127,  127,
  127,  127,  127,  146,  146,  146,  146,  146,  147,  147,
  147,  147,  147,  147,  147,  112,  148,  111,  111,  111,
  149,  149,  149,  113,  113,  113,  113,   51,   51,   51,
  150,  150,  150,   55,   55,   56,   56,   57,   70,   70,
   70,   70,   70,   48,   53,   53,   53,   53,  151,  151,
   52,   52,   49,   49,   50,   50,  116,   68,   68,   68,
};
short yylen[] = {                                         2,
    0,    1,    1,    2,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    2,    2,    2,
    2,    2,    2,    2,    2,    0,    2,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    0,    2,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    0,    2,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    0,    2,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    2,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    0,
    2,    1,    1,    1,    1,    1,    1,    0,    2,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    0,    2,    1,    1,    1,    1,    1,    7,    4,
    8,    4,    0,    3,    4,    2,    3,    3,    5,    4,
    6,    4,    3,    3,    3,    3,    1,    2,    4,    5,
    5,    5,    1,    2,    5,    7,    0,    2,    1,    1,
    1,    1,    1,    4,    1,    2,    2,    2,    2,    3,
    3,    2,    1,    1,    2,    2,    2,    2,    2,    2,
    0,    2,    0,    1,    5,    4,    2,    3,    3,    3,
    0,    4,    0,    5,    0,    5,    0,    5,    2,    1,
    2,    1,    0,    1,    4,    4,    4,    4,    4,    4,
    2,    4,    4,    3,    4,    3,    4,    1,    2,    1,
    2,    1,    1,    2,    2,    2,    2,    0,    2,    2,
    0,    2,    0,    2,   11,   10,   10,   10,    0,    2,
    2,    2,    0,    2,    4,    2,    0,    2,    0,    1,
    1,    1,    1,    0,    3,    2,    2,    2,    2,    3,
    2,    2,    2,    2,    2,    4,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    3,    1,
    1,    2,    2,    2,    0,    2,    2,    0,    2,    0,
    3,    5,    0,    2,    2,    1,    3,    4,    4,    4,
    4,    4,    4,    4,    4,    4,    4,    6,    6,    0,
    2,    0,    3,    0,    3,    7,    7,    0,    3,    1,
    1,    6,    3,    7,    1,    2,    2,    1,    2,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    0,
    4,    1,    2,    1,    1,    1,    3,    2,    2,    2,
    2,    2,    1,    2,    1,    6,    1,    2,    1,    1,
    1,    1,    6,    1,    2,    1,    1,    1,    4,    4,
    4,    1,    1,    1,    4,    4,    4,    4,    3,    3,
    1,    1,    1,    1,    1,    9,    6,    0,    1,    1,
    1,    1,    1,    0,    1,    1,    1,    1,    1,    2,
    3,    1,    1,    1,    1,    1,    1,    0,    2,    3,
    0,    2,    3,    0,    6,    6,    6,    8,   10,    7,
    0,    1,    1,    8,    9,    9,    9,    6,    0,    1,
    1,    1,    1,    8,    7,    7,    4,    4,    0,    1,
    9,    6,    8,    8,    8,    8,    2,    0,    2,    3,
};
short yydefred[] = {                                      0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    3,    5,    6,    7,    8,    9,   10,   11,   12,
   13,   14,   15,   16,   17,   26,   41,   58,   74,    0,
  100,  108,  122,    0,    0,    0,    0,  136,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  211,    0,    0,    4,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   90,
   91,   92,   94,   95,   96,   97,   98,   99,   88,   93,
    0,  325,    0,    0,    0,    0,  137,    0,    0,    0,
  153,    0,  143,  144,    0,  145,  326,    0,    0,    0,
    0,    0,  147,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   27,   28,   29,   30,
   31,   32,   33,   34,   35,   36,   37,   38,   39,   40,
    0,    0,   43,   44,   46,   48,   47,   55,   56,   57,
   42,   45,   49,   50,   51,   52,   53,   54,    0,   60,
   61,   62,   64,   63,   68,   69,   70,   65,   66,   67,
   59,   71,   72,   73,    0,   76,   77,   79,   81,   80,
   85,   86,   87,   78,   82,   83,   84,   75,    0,    0,
  222,  223,    0,  218,    0,  327,    0,  358,  359,  360,
  361,  365,    0,  363,   89,    0,    0,    0,    0,  102,
  103,  104,  105,  106,  101,  107,  110,  111,  113,  115,
  114,  119,  120,  121,  112,  116,  117,  118,  109,    0,
  124,  125,  126,  127,  128,  123,    0,    0,  132,    0,
    0,    0,    0,    0,  154,  157,  440,  441,  442,  443,
  135,  354,  355,  356,    0,  352,    0,  142,    0,  148,
    0,    0,  332,  347,  331,  345,  349,  337,  339,  348,
  342,  346,  330,  335,  338,  341,  340,  334,  333,  343,
  344,  336,    0,  328,    0,  130,  205,  206,  208,  207,
  209,  210,  212,  213,    0,    0,    0,    0,  391,  392,
  393,    0,    0,    0,    0,  367,  369,  370,  371,  372,
    0,    0,    0,    0,    0,  450,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  214,    0,  219,  220,  216,    0,  357,  364,    0,
    0,  323,  382,  383,  384,  377,  376,    0,  374,  378,
    0,  139,    0,    0,  230,  229,    0,    0,    0,    0,
  150,    0,  151,  353,    0,  253,  250,  251,  252,    0,
    0,    0,    0,  329,    0,    0,    0,  403,  399,  400,
  401,  402,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  368,    0,  432,  433,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  240,
    0,    0,    0,    0,    0,    0,    0,  224,  226,  225,
  227,  215,  221,  217,    0,    0,    0,  375,    0,  134,
    0,  232,    0,  315,  234,    0,    0,    0,    0,    0,
    0,  158,  159,  160,  161,  162,    0,    0,  173,  174,
  141,    0,  149,  351,    0,    0,  460,  242,    0,    0,
  389,    0,  394,    0,    0,    0,    0,    0,    0,    0,
  419,    0,    0,  395,  390,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  448,  447,
    0,    0,    0,    0,  457,    0,    0,    0,    0,  241,
    0,    0,  313,    0,    0,    0,    0,    0,  155,    0,
    0,    0,    0,    0,    0,  179,  167,  176,  169,  178,
  166,  168,  175,  177,    0,    0,  172,    0,    0,  248,
    0,  129,  246,  244,    0,    0,    0,    0,    0,    0,
  281,  386,  385,  388,  387,  379,  380,    0,    0,  420,
  413,  414,  409,  412,    0,  415,    0,  416,    0,    0,
    0,    0,    0,    0,    0,  405,  406,  407,  408,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  311,    0,  320,  321,    0,    0,    0,    0,    0,
  322,    0,  381,    0,    0,    0,    0,    0,    0,    0,
  170,  171,  180,    0,    0,  131,    0,  283,  284,  256,
    0,    0,  259,  271,  270,  261,  262,  267,  268,  265,
  257,  272,  258,  263,  264,  273,  274,  275,  276,  269,
    0,  277,  278,    0,    0,  282,  308,  309,    0,    0,
  366,    0,    0,    0,    0,    0,    0,    0,    0,  397,
  452,    0,    0,    0,    0,  438,    0,    0,    0,    0,
    0,  319,    0,    0,  317,  316,  324,  373,  156,    0,
    0,    0,    0,  202,    0,  200,  204,  186,  182,  184,
  164,  245,    0,  260,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  296,    0,    0,    0,  279,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  430,    0,  446,  445,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  185,  201,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  287,
  286,    0,  266,  295,    0,    0,    0,    0,  453,  454,
  455,  456,  444,    0,  428,    0,    0,    0,  434,    0,
    0,    0,    0,    0,    0,    0,  291,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  297,  289,    0,
    0,    0,    0,    0,    0,  396,  451,  435,  437,  436,
  238,  236,    0,    0,  294,  298,  299,  300,  301,  302,
  303,  304,  305,  306,  307,  237,  422,    0,    0,    0,
    0,  429,  235,  292,  423,  417,  425,  426,  427,
};
short yydgoto[] = {                                      10,
   11,   12,   13,   14,   15,   16,   17,   18,   19,   20,
   21,   22,   23,   24,   25,   26,   57,   27,   58,   28,
   59,   29,   60,   30,   69,   31,   83,   32,   84,   33,
   85,  117,   70,   71,  120,  121,   72,  123,   73,   74,
   75,   76,   77,   78,  130,  141,  142,  143,  144,  145,
  146,  147,  148,  161,  162,  163,  164,  178,   79,   80,
  205,  206,  219,  226,   81,  273,  107,  276,  229,  241,
  102,  103,  234,  360,  443,   90,  351,  245,  352,   91,
  232,  348,  432,  433,  434,  435,  436,  437,  438,  519,
  595,  671,  439,  440,  668,  665,  589,  660,  661,  662,
  666,  183,  322,  323,  184,  325,  317,  405,  450,  529,
  381,  547,  631,  489,  367,  402,  487,  685,  674,  688,
  733,  690,  530,  531,  757,  686,  373,  576,   82,  274,
  246,  193,  194,  295,  296,  297,  298,  299,  300,  338,
  339,  340,  341,  454,  301,  560,  548,  807,  772,  389,
  307,
};
short yysindex[] = {                                   1275,
 1527, -280, -263, -257, -254, -250, -248, -243, -136,    0,
 1275,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, -193,
    0,    0,    0, -224, -228, -189, -139,    0, -101,  -95,
  -98,  -28,  -64,  164, -163, -122, -117, -240,   52,   62,
   87,  116,    0,  121,  135,    0,  440, -234,  482, 1085,
 -239, -215, -215,   80,  104,  155,  108,  140, -193,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 -114,    0,  -50, 1085,  -47,  200,    0,  202, -267, -264,
    0,  151,    0,    0,   93,    0,    0,  137,  175,  154,
  207, -122,    0,  406,  418, 1471,  254,  247,  249,  252,
  269,  290,  275,  297,  308, 1405,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 -237,  234,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, -226,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, -232,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, -253,  351,
    0,    0, -266,    0, -266,    0,  375,    0,    0,    0,
    0,    0,  140,    0,    0,  559, 1471,  254, 1010,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  354,
    0,    0,    0,    0,    0,    0,  357,  430,    0,  374,
  -67,  314,  379,  431,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, -115,    0,  338,    0,  131,    0,
  137, 1471,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, -108,    0,  383,    0,    0,    0,    0,    0,
    0,    0,    0,    0, -268,  -79,  -79,  -79,    0,    0,
    0,  -79,  -79,    1, 1480,    0,    0,    0,    0,    0,
  -79, -238,  -79, -242,  -79,    0,  313,  318,   24,  296,
  -79,  -79,  343,  405,  448,  300,  453,  407,  408, -103,
    2,    0,  -39,    0,    0,    0,  -39,    0,    0, 1471,
 1191,    0,    0,    0,    0,    0,    0, 1313,    0,    0,
  -79,    0,  409,  355,    0,    0,  432,  131,  356,  439,
    0,  400,    0,    0,  435,    0,    0,    0,    0,  370,
 -111, 1235,  477,    0,  497,  449,  491,    0,    0,    0,
    0,    0,  450, -269,  -30,  459,  461,  463,  463,  -54,
  537,    0,  -16,    0,    0,  102,  119,  465,  501,  467,
  463,  463,  505,  472,  180,  209,  463,  479,   78,    0,
  410,  543,  401,  494,  491,  553,  451,    0,    0,    0,
    0,    0,    0,    0, 1310,  477,  592,    0,  506,    0,
  314,    0,  370,    0,    0,  -57,  515, -225, -198,  302,
  327,    0,    0,    0,    0,    0,   68,  578,    0,    0,
    0,  521,    0,    0,  477,  254,    0,    0,  276,  141,
    0,  513,    0,   93,  514,   93,  520,  541,  607,  632,
    0,  601, -162,    0,    0,  463,  463,  463,  463,  463,
 -230,  428,  633,  635,  463,   93,  463,  463,    0,    0,
  636,  264,  548,   93,    0,  550,  207,  192,  207,    0,
  141,  642,    0,  477,  254, -162,  554,  131,    0,  563,
  564,  565,  863,    0,  583,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  584,  585,    0,  586,  646,    0,
  254,    0,    0,    0,  131,  590,  595, 1626,  207,  365,
    0,    0,    0,    0,    0,    0,    0, -162, -162,    0,
    0,    0,    0,    0, -215,    0,  674,    0,  676,  679,
  680,  684,  686,  608,  687,    0,    0,    0,    0,  463,
 -162, -162,  689,  463,  703,  704, -162,  463,  707,  463,
  463,    0,  463,    0,    0,  622,  463,  207,  152,  254,
    0,  674,    0,  370,    0,    0,    0,  628,  629,  701,
    0,    0,    0,  639,  716,    0,  370,    0,    0,    0,
  598,  898,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  566,    0,    0,  610,  463,    0,    0,    0, -271,  729,
    0, -162, -162, -162, -162, -162,  463, -162,  743,    0,
    0, -162,  746, -162, -162,    0,  747, -162,  748,  749,
  751,    0,  758,  463,    0,    0,    0,    0,    0,  629,
  629,  629,  701,    0,  629,    0,    0,    0,    0,    0,
    0,    0,  693,    0,  -79,  -79,  -79,  -79,  -79,  -79,
  -79,  -79,  -79,  -79,  774,    0,  173,  596,  697,    0,
  778,  274,  165,  674,  674,  674,  674,  674,  783,  674,
 -162,    0, -162,    0,    0, -162,  674, -162, -162, -162,
 -162,  785,  629,  629,  629,    0,    0, -274,  698,  699,
  700,  708,  710,  711,  727,  728,  730,  731,  898,    0,
    0,  736,    0,    0, -162,  737,  737,  737,    0,    0,
    0,    0,    0, -162,    0,  674,  674,  674,    0,  674,
  674,  674,  674, -162,  715,  717,    0,  738,  739,  740,
  771,  777,  779,  812,  813,  815,  816,    0,    0,  674,
   -8,  789,  817,  826,  674,    0,    0,    0,    0,    0,
    0,    0,  674,  773,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  805, -162, -162,
 -162,    0,    0,    0,    0,    0,    0,    0,    0,
};
short yyrindex[] = {                                   1095,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1166,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  161,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  191,  547,  781,  974,
    0,    0,    0,    0,    0,    0,    0,    0, 1052,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   39,    0, 1082, 1093, 1118, 1126,    0, 1149,  438,  741,
    0,    0,    0,    0, 1159,    0,    0,    0,    0,    0,
    5, 1167,    0,    0,    0,    0, 1188,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  765,  765,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  886,    0,
    0,    0,  237,    0,  237,    0,    0,    0,    0,    0,
    0,    0,  278,    0,    0,    0,    0,  323,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  510,    0, 1204,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1215,    0,   13,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  886,  838,   96,   96,    0,    0,
    0,  838,  838,    0,  927,    0,    0,    0,    0,    0,
   96,  875,  838,  875,  838,    0,    0,    0,    0,    0,
  838,   96,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  237,    0,    0,    0,  237,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  927,    0,    0,
  838,    0,    0,  438,    0,    0,    0,  518,    0,    0,
    0, 1232,    0,    0,    0,    0,    0,    0,    0,   30,
    0,    0,  161,    0,  353,    0,  197,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  927,  927,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  927,  927,    0,    0,    0,    0,  927,    0,    0,    0,
    0, -218, -201,    0,  197,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   39,    0,    0,    0,    0,
  510,    0,  311,    0,    0,    0,    0,    0,    0,  913,
    0,    0,    0,    0,    0,    0,  791,    0,    0,    0,
    0,    0,    0,    0,  161, 1188,    0,    0,    0, -166,
    0, -160,    0, 1555, -160, 1555,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  927,  927,  927,  927,  927,
  927, -214,    0,    0,  927, -170,  927,  927,    0,    0,
    0,  927, -212, -170,    0,    0,   -5,    0,   -5,    0,
 -166,    0,    0,   39,  323,    0,    0,  518,    0,    0,
    0,    0,  859,  661,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  702,    0,
 1188,    0,    0,    0, -233,    0,    0,    0,   -5, -105,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  593,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  927,
    0,    0,    0,  927,    0,    0,    0,  927,    0,  927,
  927,    0,  927,    0,    0,    0,  927,   -5,    0,  323,
    0,  631,    0,  311,  750,  770,  811,    0,    0,  900,
    0,    0,    0,    0,  920,    0,  346,    0,    0,    0,
  -53,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 -244,    0,    0,    0,  927,    0,    0,    0,  126,    0,
    0,    0,    0,    0,    0,    0,  927,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  927,    0,    0,    0,    0,    0,    0,
    0,    0,  900,    0,  933,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  838,  838,  838,  838,  838,  838,
  838,  838,  838,  838,  -41,    0,    0,  -13,    0,    0,
    0,  203,    0,  398,  398,  398,  398,  398,    0,  398,
    0,    0,    0,    0,    0,    0,  593,    0,    0,    0,
    0,    0,  965, 1009, 1041,    0,    0,  -11,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  934,  934,  934,    0,    0,
    0,    0,    0,    0,    0,  398,  398,  593,    0,  593,
  593,  593,  593,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  593,
    0,    0,    0,    0,  398,    0,    0,    0,    0,    0,
    0,    0,  593,  -11,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,
};
short yygindex[] = {                                      0,
    0, 1181,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  502,  665,    0,    0, 1137,    0,  709,  780,
  892,  764,  930, 1039,    0,    0,    3,  205,  220,  349,
    0,    0,    0,    0,    0,    0,    0,    0, 1127,    0,
    0,    0,    0,    0, 1196, -146,  -60, -173,    0, -186,
    0, 1096, -100, -328, -412,    0,    0,  954,    0, 1116,
  864,  786,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  555,  -42,    0,    0,    0,    0,
 -355,  -63, -158, -182, -181, -311,    0,    0,  806,  719,
 -330, 1115, 1018,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  713,  433,  487, -207,    0,    0, -255,
 -195,    0, 1035,    0,  935,    0, -180,    0, -165,    0,
  893,    0,    0,  -99,    0,    0, -174, -106,  -32,  929,
 1105,
};
#define YYTABLESIZE 1964
short yytable[] = {                                     185,
  249,  324,  327,  324,  314,  106,  505,  417,  197,  353,
  499,  413,  249,  444,  285,  413,  363,  364,  336,  423,
  198,  179,  131,  302,  332,  249,  326,  314,  309,  247,
  230,  384,  385,  337,  309,  384,  385,  755,  350,  366,
  310,   86,  314,  319,  404,  233,  395,  459,  460,  354,
  331,  108,  249,  249,  109,  507,  554,  318,   62,   63,
  473,  474,  174,   61,   48,  285,  481,  181,  182,  249,
   43,   64,  181,  182,   89,  364,  249,  452,   65,  374,
  375,   49,  509,  508,  376,  377,  215,   50,  439,  453,
   51,  310,  254,  383,   52,  390,   53,  394,  395,   62,
   63,   54,  285,  398,  399,  362,  364,  180,  318,  180,
  510,   43,   64,  249,  180,   87,  380,  756,   88,   65,
  180,  320,  321,  181,  182,  410,  320,  321,  310,  231,
  541,  542,  404,  419,  395,  549,  550,  551,  552,  553,
  555,   66,  543,  254,  563,  318,  565,  566,  544,   89,
   67,  569,   68,  280,  386,  387,  391,  336,   92,  364,
  350,  303,  304,  316,  412,  354,  303,  313,  414,  584,
  315,  659,  337,  310,  392,  249,  439,  388,  316,  305,
  254,  306,   66,  415,  672,  132,  395,  311,  312,  104,
   18,   67,  104,   68,  285,  253,  597,  395,  395,  395,
  395,  254,  411,  255,  280,  290,  199,  395,   55,  220,
  395,  395,  395,  395,  395,  500,  501,  255,  502,  395,
  395,  395,  395,  395,   99,  100,  242,  243,  244,  639,
  242,  243,  244,  643,  461,  408,  312,  647,   93,  649,
  650,  280,  651,  409,   94,  288,  653,  293,   95,  395,
  395,  395,  395,  314,   43,   64,  290,   43,   64,  545,
  546,  256,   65,  158,  175,   65,  314,  533,  255,  535,
  319,  345,  522,  346,  249,  456,   97,  362,  159,  176,
  797,  503,  504,  465,  257,  258,   98,  259,  216,  564,
  260,  247,  462,  290,  691,  350,  288,  571,  293,  484,
  350,  105,  446,  217,  196,  255,  699,  261,  262,  717,
  247,   96,  263,  264,  101,  265,  455,  266,  267,  268,
  368,  581,  458,  712,  269,  270,  271,  272,  453,  350,
  464,  350,  350,  288,   67,  293,   68,   67,  798,   68,
  410,  314,  453,  350,  350,  515,  110,  596,  411,  516,
  517,  350,  459,  320,  321,  495,  111,  717,  717,  717,
  369,  370,  371,  372,  314,  314,  314,  314,  314,  314,
  314,  314,  249,  249,  249,  249,  249,  249,  249,  249,
  413,  112,  410,  410,  521,  113,  573,  410,  577,  247,
  247,  247,  247,  247,  247,  247,  247,  424,  350,  350,
  350,  350,  350,  350,  350,  350,  657,  160,  177,  314,
  314,  314,  314,  314,  350,  114,  386,  387,  410,  410,
  186,  249,  350,  350,  483,  350,  526,  527,  625,  115,
  410,  410,  218,  580,  378,  379,  453,  228,  410,  388,
  466,  314,  398,  528,  541,  542,  692,  324,  467,  249,
  187,  350,   18,  191,  398,  243,  543,  468,  350,  411,
  411,  655,  544,  227,  411,  469,  247,  719,  720,  721,
  722,  723,  724,  725,  726,  727,  728,  654,  242,  243,
  244,  629,  243,  243,  192,  410,  410,  410,  410,  410,
  410,  410,  410,  312,  228,  411,  411,  236,  312,  243,
  248,  410,  237,  238,  239,  240,  243,  411,  411,  231,
  410,  730,  410,  731,  247,  411,  233,  249,  477,  478,
  350,  350,  350,  350,  350,  350,  350,  350,  251,  312,
  312,  188,  189,  190,  362,  356,  357,  358,  359,  362,
  252,  312,  312,  243,  275,  410,   19,  479,  480,  312,
   18,   18,   18,   18,   18,   18,   18,   18,  118,  133,
  150,  166,  411,  411,  411,  411,  411,  411,  411,  411,
  362,  362,  247,  545,  546,  523,  524,  525,  411,  458,
  511,  512,  362,  362,  458,  207,  277,  411,  278,  411,
  362,  279,  424,  736,  737,  738,  312,  312,  312,  312,
  312,  312,  312,  312,  247,  513,  514,  247,  280,  459,
  380,  568,  312,  282,  459,  458,  458,  713,  714,  715,
  247,  312,  411,  312,  806,  806,  806,  458,  458,  281,
  424,  247,  247,  574,  575,  458,  283,  362,  362,  362,
  362,  362,  362,  362,  362,  459,  459,  284,  247,  247,
  526,  527,  306,  362,  424,  247,  312,  459,  459,  424,
  187,  318,  362,  328,  362,  459,  320,  321,  426,  427,
  247,  247,  247,  247,  247,  247,  247,  247,  428,  429,
  430,  330,  458,  458,  458,  458,  458,  458,  458,  458,
  424,  424,  247,  808,  809,  342,  116,  362,  458,  228,
  180,  181,  424,  424,  773,  774,  343,  458,  431,  458,
  424,  347,  459,  459,  459,  459,  459,  459,  459,  459,
  344,  119,  134,  151,  167,  349,  355,  350,  459,  365,
  395,  397,   62,   63,  228,  396,  403,  459,  149,  459,
  314,  391,  458,  400,   43,   64,  401,  228,  208,  188,
  404,  231,   65,  406,  407,  420,  424,  424,  424,  424,
  424,  424,  424,  424,  424,  124,  135,  152,  168,  189,
  422,  231,  459,  424,   62,   63,  228,  425,  442,  249,
   20,  441,  424,  104,  424,  447,   43,   64,  448,  449,
  163,  200,  209,  221,   65,  463,  451,  228,  228,  228,
  228,  228,  228,  228,  228,  457,  231,  458,   19,  380,
  190,  470,  471,  472,  249,   66,  475,  424,  476,  231,
  127,  138,  155,  171,   67,  482,   68,  249,  556,  557,
  558,  559,  486,  488,  485,  228,  125,  136,  153,  169,
  490,  492,  228,  228,  228,  228,  228,  212,  231,  424,
  496,  493,  497,  506,  424,  518,  249,   66,  199,  520,
  532,  534,  201,  210,  222,  538,   67,  536,   68,  231,
  231,  231,  231,  231,  231,  231,  231,  249,  249,  249,
  249,  249,  249,  249,  249,  424,  424,  424,  537,  540,
  539,  561,  424,  562,  567,  570,  572,  424,  424,  203,
  579,  583,  585,  586,  587,  424,   19,   19,   19,   19,
   19,   19,   19,   19,  231,  231,  231,  231,  231,  183,
  588,  590,  187,  591,  592,  593,  249,  594,  598,  187,
  187,  630,  192,  599,  632,  424,  424,  633,  634,  187,
  187,  187,  635,  424,  636,  638,  637,  642,  126,  137,
  154,  170,  424,  424,  424,  424,  424,  424,  424,  424,
  652,  644,  645,  181,  194,  648,  663,  664,  424,  187,
  181,  181,  667,   21,  202,  211,  223,  424,  669,  424,
  181,  181,  181,  673,  181,  693,  128,  139,  156,  172,
  424,  424,  424,  424,  424,  424,  424,  424,  670,  191,
  689,  701,  314,  687,  703,  706,  708,  709,  196,  710,
  181,  188,  203,  213,  224,  424,  711,  424,  188,  188,
  187,  187,  187,  187,  187,  187,  187,  187,  188,  188,
  188,  189,  718,  729,  732,  734,  735,  314,  189,  189,
  198,  744,   20,  754,  758,  759,  760,  799,  189,  189,
  189,   22,  163,  784,  761,  785,  762,  763,  188,  163,
  163,  181,  181,  181,  181,  181,  181,  181,  181,  163,
  163,  163,  190,  764,  765,  800,  766,  767,  189,  190,
  190,   23,  769,  771,  801,  786,  787,  788,  193,  190,
  190,  190,   24,  805,    1,  129,  140,  157,  173,  163,
  314,  314,  314,  314,  314,  314,  314,  314,  195,  188,
  188,  188,  188,  188,  188,  188,  188,   25,  789,  190,
  199,  204,  214,  225,  790,  138,  791,  199,  199,  189,
  189,  189,  189,  189,  189,  189,  189,  199,  199,  199,
   20,   20,   20,   20,   20,   20,   20,   20,  133,  197,
  163,  163,  163,  163,  163,  163,  163,  163,  439,  792,
  793,  203,  794,  795,  756,    2,  146,  199,  203,  203,
  190,  190,  190,  190,  190,  190,  190,  190,  203,  203,
  203,  183,  449,  239,  398,  418,  431,  458,  183,  183,
  165,   56,  421,  122,  192,  195,   47,  250,  183,  183,
  183,  192,  192,  233,  361,  235,  498,  421,  203,  578,
  491,  192,  192,  192,  140,  768,  804,  716,  199,  199,
  199,  199,  199,  199,  199,  199,  194,  329,  183,  382,
  418,  152,  393,  194,  194,   21,  308,    0,    0,    0,
    0,  192,  626,  194,  194,  194,  675,  676,  677,  678,
  679,  680,  681,  682,  683,  684,    0,    0,    0,  203,
  203,  203,  203,  203,  203,  203,  203,    0,    0,    0,
  196,    0,    0,  194,    0,    0,    0,  196,  196,  183,
  183,  183,  183,  183,  183,  183,  183,  196,  196,  196,
    0,    0,  192,  192,  192,  192,  192,  192,  192,  192,
    0,    0,  198,    0,    0,    0,    0,    0,    0,  198,
  198,    0,    0,   22,    0,  416,    0,  196,    0,  198,
  198,  198,    0,    0,  194,  194,  194,  194,  194,  194,
  194,  194,    0,   21,   21,   21,   21,   21,   21,   21,
   21,  165,    0,   23,    0,    0,    0,    0,    0,  198,
    0,    0,    0,    0,   24,    0,  180,    0,    0,  445,
    0,    0,    0,    0,    0,    0,    0,    0,  196,  196,
  196,  196,  196,  196,  196,  196,    0,   62,   63,   25,
  287,  288,  289,  290,  291,    0,    0,  138,    0,   43,
   64,  333,  334,  335,    0,    0,    0,   65,    0,    0,
  198,  198,  198,  198,  198,  198,  198,  198,    0,    0,
  133,   22,   22,   22,   22,   22,   22,   22,   22,    0,
  439,    0,    0,    0,    0,    0,    0,    0,  146,    0,
    0,    0,    0,    0,  494,    0,    0,    0,    0,    0,
    0,   23,   23,   23,   23,   23,   23,   23,   23,  458,
    0,    0,   24,   24,   24,   24,   24,   24,   24,   24,
   66,    0,    0,    0,    0,  233,    0,    0,    0,   67,
    0,   68,    0,    0,    0,    0,  140,   25,   25,   25,
   25,   25,   25,   25,   25,  138,  138,  138,  138,  138,
  138,  138,  138,  152,  253,    0,    0,    0,    0,    0,
  254,    0,  255,    0,    0,    0,    0,    0,  133,  133,
  133,  133,  133,  133,  133,  133,    0,    0,  439,  439,
  439,  439,  439,  439,  439,  439,  146,  146,  146,  146,
  146,  146,  146,  146,    0,    0,    1,    0,  253,    0,
    0,    0,    0,    0,  254,    0,  255,  458,  458,  458,
  458,  458,  458,  458,  458,    0,    0,    0,    0,    0,
  256,    0,    0,  233,  233,  233,  233,  233,  233,  233,
  233,    0,    0,    0,  140,  140,  140,  140,  140,  140,
  140,  140,    0,  257,  258,    0,  259,    0,    0,  260,
    0,  152,  152,  152,  152,  152,  152,  152,  152,  658,
    0,    0,    0,    0,  256,    0,  261,  262,    0,    0,
  582,  263,  264,  253,  265,    0,  266,  267,  268,  254,
    0,  255,    0,  269,  270,  271,  272,  257,  258,    0,
  259,    0,    0,  260,    2,    3,    4,    5,    6,    7,
    8,    9,    0,    0,    0,    0,    0,    0,    0,    0,
  261,  262,  627,  628,    0,  263,  264,    0,  265,  380,
  266,  267,  268,    0,    0,  285,    0,  269,  270,  271,
  272,    0,    0,    0,    0,  640,  641,    0,    0,  256,
    0,  646,    0,  287,  288,  289,  290,  291,    0,    0,
    0,    0,    0,  656,  333,  334,  335,    0,    0,    0,
    0,    0,  257,  258,    0,  259,    0,    0,  260,    0,
    0,  739,  740,  741,  742,  743,    0,  745,    0,    0,
    0,    0,    0,    0,  749,  261,  262,    0,    0,    0,
  263,  264,    0,  265,    0,  266,  267,  268,    0,    0,
    0,    0,  269,  270,  271,  272,  694,  695,  696,  697,
  698,  180,  700,    0,    0,    0,  702,    0,  704,  705,
    0,    0,  707,  776,  777,  778,    0,  779,  780,  781,
  782,    0,  286,    0,  253,  287,  288,  289,  290,  291,
  254,    0,  255,    0,  292,  293,    0,  796,    0,   34,
    0,   35,  802,   36,   37,    0,    0,   38,    0,    0,
  803,    0,    0,    0,  294,    0,    0,    0,    0,    0,
   39,   40,    0,  439,   41,  746,    0,  747,   42,    0,
  748,    0,  750,  751,  752,  753,  380,    0,    0,    0,
    0,   43,    0,   44,    0,    0,    0,    0,    0,    0,
  256,    0,    0,    0,    0,    0,    0,  286,    0,  770,
  287,  288,  289,  290,  291,    0,    0,    0,  775,  292,
  293,    0,    0,  257,  258,    0,  259,    0,  783,  260,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  261,  262,    0,    0,
    0,  263,  264,    0,  265,    0,  266,  267,  268,    0,
    0,  439,    0,  269,  270,  271,  272,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   45,    0,    0,    0,
    0,   46,  439,    0,    0,  439,  439,  439,  439,  439,
    0,    0,    0,    0,  439,  439,  439,  439,  439,  600,
  601,  602,  603,  604,  605,  606,  607,  608,  609,  610,
  611,  612,  613,  614,  615,  616,  617,  618,  619,  620,
  621,  622,  623,  624,
};
short yycheck[] = {                                      63,
  101,  183,  185,  185,    0,  123,   64,  338,  123,  125,
  423,  323,    0,  125,  259,  327,  125,  273,  199,  348,
   81,  261,  257,  261,  198,  259,  185,  296,  261,    0,
  298,  274,  275,  199,  261,  274,  275,  312,    0,  308,
  259,  266,  296,  310,  259,  310,  259,  378,  379,  245,
  197,  292,  286,  287,  295,  281,  287,  259,  293,  294,
  391,  392,   60,  257,  345,  310,  397,  339,  340,  303,
  305,  306,  339,  340,  339,  331,  310,  347,  313,  287,
  288,  345,  281,  309,  292,  293,   84,  345,  259,  359,
  345,  310,  259,  301,  345,  303,  345,  305,  259,  293,
  294,  345,  347,  311,  312,  252,  362,  347,  310,  347,
  309,  305,  306,  347,  347,  340,  347,  392,  347,  313,
  347,  393,  394,  339,  340,    0,  393,  394,  347,  397,
  293,  294,  347,  341,  347,  466,  467,  468,  469,  470,
  471,  376,  305,  310,  475,  347,  477,  478,  311,  339,
  385,  482,  387,  259,  393,  394,  399,  338,  298,  415,
    0,  399,  400,  432,  323,  361,  399,  400,  327,  498,
  424,  584,  338,  400,  417,  409,  347,  416,  432,  417,
  347,  419,  376,  330,  597,  420,  347,  414,  415,  307,
    0,  385,  307,  387,  439,  304,  525,  410,  411,  412,
  413,  310,    0,  312,  310,  259,  257,  368,  345,  257,
  371,  372,  373,  374,  375,  273,  274,  259,  276,  380,
  381,  382,  383,  384,  388,  389,  342,  343,  344,  560,
  342,  343,  344,  564,  289,  339,    0,  568,  340,  570,
  571,  347,  573,  347,  340,  259,  577,  259,  347,  410,
  411,  412,  413,  259,  305,  306,  310,  305,  306,  422,
  423,  370,  313,   59,   60,  313,  262,  454,  310,  456,
  310,  339,  446,  341,  262,  375,  341,    0,   59,   60,
  289,  339,  340,  383,  393,  394,  123,  396,   84,  476,
  399,  262,  347,  347,  625,  257,  310,  484,  310,  399,
  262,  419,  363,   84,  419,  347,  637,  416,  417,  665,
    0,  340,  421,  422,  437,  424,  347,  426,  427,  428,
  400,  495,    0,  654,  433,  434,  435,  436,  359,  291,
  347,  293,  294,  347,  385,  347,  387,  385,  347,  387,
  339,  347,  359,  305,  306,  278,  295,  521,  347,  282,
  283,  313,    0,  393,  394,  416,  295,  713,  714,  715,
  440,  441,  442,  443,  360,  361,  362,  363,  364,  365,
  366,  367,  360,  361,  362,  363,  364,  365,  366,  367,
  692,  295,  257,  258,  445,  270,  487,  262,  489,  360,
  361,  362,  363,  364,  365,  366,  367,    0,  360,  361,
  362,  363,  364,  365,  366,  367,  580,   59,   60,  405,
  406,  407,  408,  409,  376,  295,  393,  394,  293,  294,
  341,  409,  262,  385,  347,  387,  286,  287,  529,  295,
  305,  306,   84,  494,  434,  435,  359,    0,  313,  416,
  339,  437,  347,  303,  293,  294,  629,  629,  347,  437,
  347,  291,  262,  346,  359,  259,  305,  339,  420,  257,
  258,  310,  311,  264,  262,  347,  437,  675,  676,  677,
  678,  679,  680,  681,  682,  683,  684,  578,  342,  343,
  344,  545,  286,  287,  345,  360,  361,  362,  363,  364,
  365,  366,  367,  257,  293,  293,  294,  347,  262,  303,
  347,  376,  410,  411,  412,  413,  310,  305,  306,    0,
  385,  339,  387,  341,  340,  313,  310,    0,  339,  340,
  360,  361,  362,  363,  364,  365,  366,  367,  123,  293,
  294,  377,  378,  379,  257,  405,  406,  407,  408,  262,
  123,  305,  306,  347,  291,  420,    0,  339,  340,  313,
  360,  361,  362,  363,  364,  365,  366,  367,   57,   58,
   59,   60,  360,  361,  362,  363,  364,  365,  366,  367,
  293,  294,  262,  422,  423,  300,  301,  302,  376,  257,
  279,  280,  305,  306,  262,   84,  340,  385,  340,  387,
  313,  340,    0,  429,  430,  431,  360,  361,  362,  363,
  364,  365,  366,  367,  259,  279,  280,  297,  340,  257,
  347,  348,  376,  339,  262,  293,  294,  660,  661,  662,
  310,  385,  420,  387,  799,  800,  801,  305,  306,  340,
    0,  286,  287,  442,  443,  313,  340,  360,  361,  362,
  363,  364,  365,  366,  367,  293,  294,  340,  303,  339,
  286,  287,  419,  376,  257,  310,  420,  305,  306,  262,
    0,  311,  385,  289,  387,  313,  393,  394,  269,  270,
  360,  361,  362,  363,  364,  365,  366,  367,  279,  280,
  281,  123,  360,  361,  362,  363,  364,  365,  366,  367,
  293,  294,  347,  800,  801,  339,  257,  420,  376,  262,
  347,    0,  305,  306,  737,  738,  277,  385,  309,  387,
  313,  398,  360,  361,  362,  363,  364,  365,  366,  367,
  347,   57,   58,   59,   60,  347,  389,  297,  376,  347,
  418,  436,  293,  294,  297,  418,  437,  385,  257,  387,
    0,  399,  420,  339,  305,  306,  299,  310,   84,    0,
  298,  397,  313,  347,  347,  347,  401,  360,  361,  362,
  363,  364,  365,  366,  367,   57,   58,   59,   60,    0,
  339,  262,  420,  376,  293,  294,  339,  339,  409,  262,
    0,  347,  385,  307,  387,  289,  305,  306,  340,  299,
    0,   83,   84,   85,  313,  259,  347,  360,  361,  362,
  363,  364,  365,  366,  367,  347,  297,  347,  262,  347,
    0,  347,  312,  347,  297,  376,  312,  420,  347,  310,
   57,   58,   59,   60,  385,  347,  387,  310,  401,  402,
  403,  404,  290,  433,  425,  398,   57,   58,   59,   60,
  347,  289,  405,  406,  407,  408,  409,   84,  339,  257,
  259,  401,  347,  339,  262,  278,  339,  376,    0,  339,
  348,  348,   83,   84,   85,  259,  385,  348,  387,  360,
  361,  362,  363,  364,  365,  366,  367,  360,  361,  362,
  363,  364,  365,  366,  367,  293,  294,  257,  348,  289,
  259,  259,  262,  259,  259,  348,  347,  305,  306,    0,
  259,  348,  340,  340,  340,  313,  360,  361,  362,  363,
  364,  365,  366,  367,  405,  406,  407,  408,  409,    0,
   58,  339,  262,  340,  340,  340,  409,  282,  339,  269,
  270,  258,    0,  339,  259,  305,  306,  259,  259,  279,
  280,  281,  259,  313,  259,  259,  339,  259,   57,   58,
   59,   60,  360,  361,  362,  363,  364,  365,  366,  367,
  339,  259,  259,  262,    0,  259,  339,  339,  376,  309,
  269,  270,  272,    0,   83,   84,   85,  385,  340,  387,
  279,  280,  281,  386,  283,  257,   57,   58,   59,   60,
  360,  361,  362,  363,  364,  365,  366,  367,  283,  339,
  391,  259,  262,  438,  259,  259,  259,  259,    0,  259,
  309,  262,   83,   84,   85,  385,  259,  387,  269,  270,
  360,  361,  362,  363,  364,  365,  366,  367,  279,  280,
  281,  262,  340,  260,  439,  339,  259,  297,  269,  270,
    0,  259,  262,  259,  347,  347,  347,  259,  279,  280,
  281,    0,  262,  339,  347,  339,  347,  347,  309,  269,
  270,  360,  361,  362,  363,  364,  365,  366,  367,  279,
  280,  281,  262,  347,  347,  259,  347,  347,  309,  269,
  270,    0,  347,  347,  259,  348,  348,  348,  339,  279,
  280,  281,    0,  289,    0,   57,   58,   59,   60,  309,
  360,  361,  362,  363,  364,  365,  366,  367,  339,  360,
  361,  362,  363,  364,  365,  366,  367,    0,  348,  309,
  262,   83,   84,   85,  348,    0,  348,  269,  270,  360,
  361,  362,  363,  364,  365,  366,  367,  279,  280,  281,
  360,  361,  362,  363,  364,  365,  366,  367,    0,  339,
  360,  361,  362,  363,  364,  365,  366,  367,    0,  348,
  348,  262,  348,  348,  392,    0,    0,  309,  269,  270,
  360,  361,  362,  363,  364,  365,  366,  367,  279,  280,
  281,  262,  418,  298,  347,  259,  312,    0,  269,  270,
  278,   11,  259,   57,  262,   69,    1,  102,  279,  280,
  281,  269,  270,    0,  251,   90,  421,  344,  309,  491,
  405,  279,  280,  281,    0,  729,  784,  663,  360,  361,
  362,  363,  364,  365,  366,  367,  262,  193,  309,  295,
  338,    0,  304,  269,  270,  262,  132,   -1,   -1,   -1,
   -1,  309,  530,  279,  280,  281,  349,  350,  351,  352,
  353,  354,  355,  356,  357,  358,   -1,   -1,   -1,  360,
  361,  362,  363,  364,  365,  366,  367,   -1,   -1,   -1,
  262,   -1,   -1,  309,   -1,   -1,   -1,  269,  270,  360,
  361,  362,  363,  364,  365,  366,  367,  279,  280,  281,
   -1,   -1,  360,  361,  362,  363,  364,  365,  366,  367,
   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,  269,
  270,   -1,   -1,  262,   -1,  125,   -1,  309,   -1,  279,
  280,  281,   -1,   -1,  360,  361,  362,  363,  364,  365,
  366,  367,   -1,  360,  361,  362,  363,  364,  365,  366,
  367,  257,   -1,  262,   -1,   -1,   -1,   -1,   -1,  309,
   -1,   -1,   -1,   -1,  262,   -1,  347,   -1,   -1,  125,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,  361,
  362,  363,  364,  365,  366,  367,   -1,  293,  294,  262,
  371,  372,  373,  374,  375,   -1,   -1,  262,   -1,  305,
  306,  382,  383,  384,   -1,   -1,   -1,  313,   -1,   -1,
  360,  361,  362,  363,  364,  365,  366,  367,   -1,   -1,
  262,  360,  361,  362,  363,  364,  365,  366,  367,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,
   -1,   -1,   -1,   -1,  125,   -1,   -1,   -1,   -1,   -1,
   -1,  360,  361,  362,  363,  364,  365,  366,  367,  262,
   -1,   -1,  360,  361,  362,  363,  364,  365,  366,  367,
  376,   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,  385,
   -1,  387,   -1,   -1,   -1,   -1,  262,  360,  361,  362,
  363,  364,  365,  366,  367,  360,  361,  362,  363,  364,
  365,  366,  367,  262,  304,   -1,   -1,   -1,   -1,   -1,
  310,   -1,  312,   -1,   -1,   -1,   -1,   -1,  360,  361,
  362,  363,  364,  365,  366,  367,   -1,   -1,  360,  361,
  362,  363,  364,  365,  366,  367,  360,  361,  362,  363,
  364,  365,  366,  367,   -1,   -1,  262,   -1,  304,   -1,
   -1,   -1,   -1,   -1,  310,   -1,  312,  360,  361,  362,
  363,  364,  365,  366,  367,   -1,   -1,   -1,   -1,   -1,
  370,   -1,   -1,  360,  361,  362,  363,  364,  365,  366,
  367,   -1,   -1,   -1,  360,  361,  362,  363,  364,  365,
  366,  367,   -1,  393,  394,   -1,  396,   -1,   -1,  399,
   -1,  360,  361,  362,  363,  364,  365,  366,  367,  582,
   -1,   -1,   -1,   -1,  370,   -1,  416,  417,   -1,   -1,
  496,  421,  422,  304,  424,   -1,  426,  427,  428,  310,
   -1,  312,   -1,  433,  434,  435,  436,  393,  394,   -1,
  396,   -1,   -1,  399,  360,  361,  362,  363,  364,  365,
  366,  367,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  416,  417,  538,  539,   -1,  421,  422,   -1,  424,  347,
  426,  427,  428,   -1,   -1,  261,   -1,  433,  434,  435,
  436,   -1,   -1,   -1,   -1,  561,  562,   -1,   -1,  370,
   -1,  567,   -1,  371,  372,  373,  374,  375,   -1,   -1,
   -1,   -1,   -1,  579,  382,  383,  384,   -1,   -1,   -1,
   -1,   -1,  393,  394,   -1,  396,   -1,   -1,  399,   -1,
   -1,  694,  695,  696,  697,  698,   -1,  700,   -1,   -1,
   -1,   -1,   -1,   -1,  707,  416,  417,   -1,   -1,   -1,
  421,  422,   -1,  424,   -1,  426,  427,  428,   -1,   -1,
   -1,   -1,  433,  434,  435,  436,  632,  633,  634,  635,
  636,  347,  638,   -1,   -1,   -1,  642,   -1,  644,  645,
   -1,   -1,  648,  746,  747,  748,   -1,  750,  751,  752,
  753,   -1,  368,   -1,  304,  371,  372,  373,  374,  375,
  310,   -1,  312,   -1,  380,  381,   -1,  770,   -1,  263,
   -1,  265,  775,  267,  268,   -1,   -1,  271,   -1,   -1,
  783,   -1,   -1,   -1,  400,   -1,   -1,   -1,   -1,   -1,
  284,  285,   -1,  259,  288,  701,   -1,  703,  292,   -1,
  706,   -1,  708,  709,  710,  711,  347,   -1,   -1,   -1,
   -1,  305,   -1,  307,   -1,   -1,   -1,   -1,   -1,   -1,
  370,   -1,   -1,   -1,   -1,   -1,   -1,  368,   -1,  735,
  371,  372,  373,  374,  375,   -1,   -1,   -1,  744,  380,
  381,   -1,   -1,  393,  394,   -1,  396,   -1,  754,  399,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  416,  417,   -1,   -1,
   -1,  421,  422,   -1,  424,   -1,  426,  427,  428,   -1,
   -1,  347,   -1,  433,  434,  435,  436,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  390,   -1,   -1,   -1,
   -1,  395,  368,   -1,   -1,  371,  372,  373,  374,  375,
   -1,   -1,   -1,   -1,  380,  381,  382,  383,  384,  314,
  315,  316,  317,  318,  319,  320,  321,  322,  323,  324,
  325,  326,  327,  328,  329,  330,  331,  332,  333,  334,
  335,  336,  337,  338,
};
#define YYFINAL 10
#ifndef YYDEBUG
#define YYDEBUG 1
#endif
#define YYMAXTOKEN 443
#if YYDEBUG
char *yyname[] = {
"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"':'",0,0,0,0,0,"'@'",0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,"'{'",0,"'}'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"IF","ELSE",
"THEN","OR","FAILED","SET","LOGFILE","FACILITY","DAEMON","SYSLOG","MAILSERVER",
"HTTPD","ALLOW","ADDRESS","INIT","READONLY","CLEARTEXT","MD5HASH","SHA1HASH",
"CRYPT","DELAY","PEMFILE","ENABLE","DISABLE","HTTPDSSL","CLIENTPEMFILE",
"ALLOWSELFCERTIFICATION","IDFILE","STATEFILE","SEND","EXPECT","EXPECTBUFFER",
"CYCLE","COUNT","REMINDER","PIDFILE","START","STOP","PATHTOK","HOST","HOSTNAME",
"PORT","TYPE","UDP","TCP","TCPSSL","PROTOCOL","CONNECTION","ALERT","NOALERT",
"MAILFORMAT","UNIXSOCKET","SIGNATURE","TIMEOUT","RESTART","CHECKSUM","EVERY",
"DEFAULT","HTTP","APACHESTATUS","FTP","SMTP","POP","IMAP","CLAMAV","NNTP",
"NTP3","MYSQL","DNS","SSH","DWP","LDAP2","LDAP3","RDATE","RSYNC","TNS","PGSQL",
"POSTFIXPOLICY","SIP","LMTP","GPS","RADIUS","STRING","PATH","MAILADDR",
"MAILFROM","MAILSUBJECT","MAILBODY","SERVICENAME","STRINGNAME","NUMBER",
"PERCENT","LOGLIMIT","CLOSELIMIT","DNSLIMIT","KEEPALIVELIMIT","REPLYLIMIT",
"REQUESTLIMIT","STARTLIMIT","WAITLIMIT","GRACEFULLIMIT","CLEANUPLIMIT","REAL",
"CHECKPROC","CHECKFILESYS","CHECKFILE","CHECKDIR","CHECKHOST","CHECKSYSTEM",
"CHECKFIFO","CHECKSTATUS","CHILDREN","SYSTEM","RESOURCE","MEMORY","TOTALMEMORY",
"LOADAVG1","LOADAVG5","LOADAVG15","MODE","ACTIVE","PASSIVE","MANUAL","CPU",
"TOTALCPU","CPUUSER","CPUSYSTEM","CPUWAIT","GROUP","REQUEST","DEPENDS",
"BASEDIR","SLOT","EVENTQUEUE","SECRET","HOSTHEADER","UID","GID","MMONIT",
"INSTANCE","USERNAME","PASSWORD","TIMESTAMP","CHANGED","SECOND","MINUTE","HOUR",
"DAY","SSLAUTO","SSLV2","SSLV3","TLSV1","CERTMD5","BYTE","KILOBYTE","MEGABYTE",
"GIGABYTE","INODE","SPACE","PERMISSION","SIZE","MATCH","NOT","IGNORE","ACTION",
"EXEC","UNMONITOR","ICMP","ICMPECHO","NONEXIST","INVALID","DATA","RECOVERED",
"PASSED","SUCCEEDED","URL","CONTENT","PID","PPID","FSFLAG","URLOBJECT","TARGET",
"MAXFORWARD","GREATER","LESS","EQUAL","NOTEQUAL",
};
char *yyrule[] = {
"$accept : cfgfile",
"cfgfile :",
"cfgfile : statement_list",
"statement_list : statement",
"statement_list : statement_list statement",
"statement : setalert",
"statement : setdaemon",
"statement : setlog",
"statement : seteventqueue",
"statement : setmmonits",
"statement : setmailservers",
"statement : setmailformat",
"statement : sethttpd",
"statement : setpid",
"statement : setidfile",
"statement : setstatefile",
"statement : setexpectbuffer",
"statement : setinit",
"statement : checkproc optproclist",
"statement : checkfile optfilelist",
"statement : checkfilesys optfilesyslist",
"statement : checkdir optdirlist",
"statement : checkhost opthostlist",
"statement : checksystem optsystemlist",
"statement : checkfifo optfifolist",
"statement : checkstatus optstatuslist",
"optproclist :",
"optproclist : optproclist optproc",
"optproc : start",
"optproc : stop",
"optproc : pid",
"optproc : ppid",
"optproc : connection",
"optproc : connectionunix",
"optproc : actionrate",
"optproc : alert",
"optproc : every",
"optproc : mode",
"optproc : group",
"optproc : depend",
"optproc : resourceprocess",
"optfilelist :",
"optfilelist : optfilelist optfile",
"optfile : start",
"optfile : stop",
"optfile : timestamp",
"optfile : actionrate",
"optfile : every",
"optfile : alert",
"optfile : permission",
"optfile : uid",
"optfile : gid",
"optfile : checksum",
"optfile : size",
"optfile : match",
"optfile : mode",
"optfile : group",
"optfile : depend",
"optfilesyslist :",
"optfilesyslist : optfilesyslist optfilesys",
"optfilesys : start",
"optfilesys : stop",
"optfilesys : actionrate",
"optfilesys : every",
"optfilesys : alert",
"optfilesys : permission",
"optfilesys : uid",
"optfilesys : gid",
"optfilesys : mode",
"optfilesys : group",
"optfilesys : depend",
"optfilesys : inode",
"optfilesys : space",
"optfilesys : fsflag",
"optdirlist :",
"optdirlist : optdirlist optdir",
"optdir : start",
"optdir : stop",
"optdir : timestamp",
"optdir : actionrate",
"optdir : every",
"optdir : alert",
"optdir : permission",
"optdir : uid",
"optdir : gid",
"optdir : mode",
"optdir : group",
"optdir : depend",
"opthostlist : opthost",
"opthostlist : opthostlist opthost",
"opthost : start",
"opthost : stop",
"opthost : connection",
"opthost : icmp",
"opthost : actionrate",
"opthost : alert",
"opthost : every",
"opthost : mode",
"opthost : group",
"opthost : depend",
"optsystemlist :",
"optsystemlist : optsystemlist optsystem",
"optsystem : actionrate",
"optsystem : alert",
"optsystem : every",
"optsystem : group",
"optsystem : depend",
"optsystem : resourcesystem",
"optfifolist :",
"optfifolist : optfifolist optfifo",
"optfifo : start",
"optfifo : stop",
"optfifo : timestamp",
"optfifo : actionrate",
"optfifo : every",
"optfifo : alert",
"optfifo : permission",
"optfifo : uid",
"optfifo : gid",
"optfifo : mode",
"optfifo : group",
"optfifo : depend",
"optstatuslist :",
"optstatuslist : optstatuslist optstatus",
"optstatus : actionrate",
"optstatus : alert",
"optstatus : every",
"optstatus : group",
"optstatus : depend",
"setalert : SET alertmail '{' eventoptionlist '}' formatlist reminder",
"setalert : SET alertmail formatlist reminder",
"setalert : SET alertmail NOT '{' eventoptionlist '}' formatlist reminder",
"setdaemon : SET DAEMON NUMBER startdelay",
"startdelay :",
"startdelay : START DELAY NUMBER",
"setexpectbuffer : SET EXPECTBUFFER NUMBER unit",
"setinit : SET INIT",
"setlog : SET LOGFILE PATH",
"setlog : SET LOGFILE SYSLOG",
"setlog : SET LOGFILE SYSLOG FACILITY STRING",
"seteventqueue : SET EVENTQUEUE BASEDIR PATH",
"seteventqueue : SET EVENTQUEUE BASEDIR PATH SLOT NUMBER",
"seteventqueue : SET EVENTQUEUE SLOT NUMBER",
"setidfile : SET IDFILE PATH",
"setstatefile : SET STATEFILE PATH",
"setpid : SET PIDFILE PATH",
"setmmonits : SET MMONIT mmonitlist",
"mmonitlist : mmonit",
"mmonitlist : mmonitlist mmonit",
"mmonit : URLOBJECT nettimeout sslversion certmd5",
"setmailservers : SET MAILSERVER mailserverlist nettimeout hostname",
"setmailformat : SET MAILFORMAT '{' formatoptionlist '}'",
"sethttpd : SET HTTPD PORT NUMBER httpdlist",
"mailserverlist : mailserver",
"mailserverlist : mailserverlist mailserver",
"mailserver : STRING username password sslversion certmd5",
"mailserver : STRING PORT NUMBER username password sslversion certmd5",
"httpdlist :",
"httpdlist : httpdlist httpdoption",
"httpdoption : ssl",
"httpdoption : signature",
"httpdoption : bindaddress",
"httpdoption : allow",
"ssl : ssldisable",
"ssl : sslenable pemfile clientpemfile allowselfcert",
"sslenable : HTTPDSSL",
"sslenable : HTTPDSSL ENABLE",
"sslenable : ENABLE HTTPDSSL",
"ssldisable : HTTPDSSL DISABLE",
"ssldisable : DISABLE HTTPDSSL",
"ssldisable : ssldisable PEMFILE PATH",
"ssldisable : ssldisable CLIENTPEMFILE PATH",
"ssldisable : ssldisable ALLOWSELFCERTIFICATION",
"signature : sigenable",
"signature : sigdisable",
"sigenable : SIGNATURE ENABLE",
"sigenable : ENABLE SIGNATURE",
"sigdisable : SIGNATURE DISABLE",
"sigdisable : DISABLE SIGNATURE",
"bindaddress : ADDRESS STRING",
"pemfile : PEMFILE PATH",
"clientpemfile :",
"clientpemfile : CLIENTPEMFILE PATH",
"allowselfcert :",
"allowselfcert : ALLOWSELFCERTIFICATION",
"allow : ALLOW STRING ':' STRING readonly",
"allow : ALLOW '@' STRING readonly",
"allow : ALLOW PATH",
"allow : ALLOW CLEARTEXT PATH",
"allow : ALLOW MD5HASH PATH",
"allow : ALLOW CRYPT PATH",
"$$1 :",
"allow : ALLOW PATH $$1 allowuserlist",
"$$2 :",
"allow : ALLOW CLEARTEXT PATH $$2 allowuserlist",
"$$3 :",
"allow : ALLOW MD5HASH PATH $$3 allowuserlist",
"$$4 :",
"allow : ALLOW CRYPT PATH $$4 allowuserlist",
"allow : ALLOW STRING",
"allowuserlist : allowuser",
"allowuserlist : allowuserlist allowuser",
"allowuser : STRING",
"readonly :",
"readonly : READONLY",
"checkproc : CHECKPROC SERVICENAME PIDFILE PATH",
"checkproc : CHECKPROC SERVICENAME PATHTOK PATH",
"checkfile : CHECKFILE SERVICENAME PATHTOK PATH",
"checkfilesys : CHECKFILESYS SERVICENAME PATHTOK PATH",
"checkdir : CHECKDIR SERVICENAME PATHTOK PATH",
"checkhost : CHECKHOST SERVICENAME ADDRESS STRING",
"checksystem : CHECKSYSTEM SERVICENAME",
"checkfifo : CHECKFIFO SERVICENAME PATHTOK PATH",
"checkstatus : CHECKSTATUS SERVICENAME PATHTOK PATH",
"start : START argumentlist exectimeout",
"start : START argumentlist useroptionlist exectimeout",
"stop : STOP argumentlist exectimeout",
"stop : STOP argumentlist useroptionlist exectimeout",
"argumentlist : argument",
"argumentlist : argumentlist argument",
"useroptionlist : useroption",
"useroptionlist : useroptionlist useroption",
"argument : STRING",
"argument : PATH",
"useroption : UID STRING",
"useroption : GID STRING",
"useroption : UID NUMBER",
"useroption : GID NUMBER",
"username :",
"username : USERNAME MAILADDR",
"username : USERNAME STRING",
"password :",
"password : PASSWORD STRING",
"hostname :",
"hostname : HOSTNAME STRING",
"connection : IF FAILED host port type protocol nettimeout rate1 THEN action1 recovery",
"connection : IF FAILED URL URLOBJECT urloption nettimeout rate1 THEN action1 recovery",
"connectionunix : IF FAILED unixsocket type protocol nettimeout rate1 THEN action1 recovery",
"icmp : IF FAILED ICMP icmptype icmpcount nettimeout rate1 THEN action1 recovery",
"host :",
"host : HOST STRING",
"port : PORT NUMBER",
"unixsocket : UNIXSOCKET PATH",
"type :",
"type : TYPE TCP",
"type : TYPE TCPSSL sslversion certmd5",
"type : TYPE UDP",
"certmd5 :",
"certmd5 : CERTMD5 STRING",
"sslversion :",
"sslversion : SSLV2",
"sslversion : SSLV3",
"sslversion : TLSV1",
"sslversion : SSLAUTO",
"protocol :",
"protocol : PROTOCOL APACHESTATUS apache_stat_list",
"protocol : PROTOCOL DEFAULT",
"protocol : PROTOCOL DNS",
"protocol : PROTOCOL DWP",
"protocol : PROTOCOL FTP",
"protocol : PROTOCOL HTTP request",
"protocol : PROTOCOL IMAP",
"protocol : PROTOCOL CLAMAV",
"protocol : PROTOCOL LDAP2",
"protocol : PROTOCOL LDAP3",
"protocol : PROTOCOL MYSQL",
"protocol : PROTOCOL SIP target maxforward",
"protocol : PROTOCOL NNTP",
"protocol : PROTOCOL NTP3",
"protocol : PROTOCOL POSTFIXPOLICY",
"protocol : PROTOCOL POP",
"protocol : PROTOCOL SMTP",
"protocol : PROTOCOL SSH",
"protocol : PROTOCOL RDATE",
"protocol : PROTOCOL RSYNC",
"protocol : PROTOCOL TNS",
"protocol : PROTOCOL PGSQL",
"protocol : PROTOCOL LMTP",
"protocol : PROTOCOL GPS",
"protocol : PROTOCOL RADIUS secret",
"protocol : sendexpectlist",
"sendexpectlist : sendexpect",
"sendexpectlist : sendexpectlist sendexpect",
"sendexpect : SEND STRING",
"sendexpect : EXPECT STRING",
"target :",
"target : TARGET MAILADDR",
"target : TARGET STRING",
"maxforward :",
"maxforward : MAXFORWARD NUMBER",
"request :",
"request : REQUEST PATH hostheader",
"request : REQUEST PATH CHECKSUM STRING hostheader",
"hostheader :",
"hostheader : HOSTHEADER STRING",
"secret : SECRET STRING",
"apache_stat_list : apache_stat",
"apache_stat_list : apache_stat_list OR apache_stat",
"apache_stat : LOGLIMIT operator NUMBER PERCENT",
"apache_stat : CLOSELIMIT operator NUMBER PERCENT",
"apache_stat : DNSLIMIT operator NUMBER PERCENT",
"apache_stat : KEEPALIVELIMIT operator NUMBER PERCENT",
"apache_stat : REPLYLIMIT operator NUMBER PERCENT",
"apache_stat : REQUESTLIMIT operator NUMBER PERCENT",
"apache_stat : STARTLIMIT operator NUMBER PERCENT",
"apache_stat : WAITLIMIT operator NUMBER PERCENT",
"apache_stat : GRACEFULLIMIT operator NUMBER PERCENT",
"apache_stat : CLEANUPLIMIT operator NUMBER PERCENT",
"pid : IF CHANGED PID rate1 THEN action1",
"ppid : IF CHANGED PPID rate1 THEN action1",
"icmpcount :",
"icmpcount : COUNT NUMBER",
"exectimeout :",
"exectimeout : TIMEOUT NUMBER SECOND",
"nettimeout :",
"nettimeout : TIMEOUT NUMBER SECOND",
"actionrate : IF NUMBER RESTART NUMBER CYCLE THEN action1",
"actionrate : IF NUMBER RESTART NUMBER CYCLE THEN TIMEOUT",
"urloption :",
"urloption : CONTENT urloperator STRING",
"urloperator : EQUAL",
"urloperator : NOTEQUAL",
"alert : alertmail '{' eventoptionlist '}' formatlist reminder",
"alert : alertmail formatlist reminder",
"alert : alertmail NOT '{' eventoptionlist '}' formatlist reminder",
"alert : noalertmail",
"alertmail : ALERT MAILADDR",
"noalertmail : NOALERT MAILADDR",
"eventoptionlist : eventoption",
"eventoptionlist : eventoptionlist eventoption",
"eventoption : ACTION",
"eventoption : CHECKSUM",
"eventoption : CONNECTION",
"eventoption : CONTENT",
"eventoption : DATA",
"eventoption : EXEC",
"eventoption : FSFLAG",
"eventoption : GID",
"eventoption : ICMP",
"eventoption : INSTANCE",
"eventoption : INVALID",
"eventoption : NONEXIST",
"eventoption : PERMISSION",
"eventoption : PID",
"eventoption : PPID",
"eventoption : RESOURCE",
"eventoption : SIZE",
"eventoption : TIMEOUT",
"eventoption : TIMESTAMP",
"eventoption : UID",
"formatlist :",
"formatlist : MAILFORMAT '{' formatoptionlist '}'",
"formatoptionlist : formatoption",
"formatoptionlist : formatoptionlist formatoption",
"formatoption : MAILFROM",
"formatoption : MAILSUBJECT",
"formatoption : MAILBODY",
"every : EVERY NUMBER CYCLE",
"mode : MODE ACTIVE",
"mode : MODE PASSIVE",
"mode : MODE MANUAL",
"group : GROUP STRINGNAME",
"depend : DEPENDS dependlist",
"dependlist : dependant",
"dependlist : dependlist dependant",
"dependant : SERVICENAME",
"resourceprocess : IF resourceprocesslist rate1 THEN action1 recovery",
"resourceprocesslist : resourceprocessopt",
"resourceprocesslist : resourceprocesslist resourceprocessopt",
"resourceprocessopt : resourcecpuproc",
"resourceprocessopt : resourcemem",
"resourceprocessopt : resourcechild",
"resourceprocessopt : resourceload",
"resourcesystem : IF resourcesystemlist rate1 THEN action1 recovery",
"resourcesystemlist : resourcesystemopt",
"resourcesystemlist : resourcesystemlist resourcesystemopt",
"resourcesystemopt : resourceload",
"resourcesystemopt : resourcemem",
"resourcesystemopt : resourcecpu",
"resourcecpuproc : CPU operator NUMBER PERCENT",
"resourcecpuproc : TOTALCPU operator NUMBER PERCENT",
"resourcecpu : resourcecpuid operator NUMBER PERCENT",
"resourcecpuid : CPUUSER",
"resourcecpuid : CPUSYSTEM",
"resourcecpuid : CPUWAIT",
"resourcemem : MEMORY operator value unit",
"resourcemem : MEMORY operator NUMBER PERCENT",
"resourcemem : TOTALMEMORY operator value unit",
"resourcemem : TOTALMEMORY operator NUMBER PERCENT",
"resourcechild : CHILDREN operator NUMBER",
"resourceload : resourceloadavg operator value",
"resourceloadavg : LOADAVG1",
"resourceloadavg : LOADAVG5",
"resourceloadavg : LOADAVG15",
"value : REAL",
"value : NUMBER",
"timestamp : IF TIMESTAMP operator NUMBER time rate1 THEN action1 recovery",
"timestamp : IF CHANGED TIMESTAMP rate1 THEN action1",
"operator :",
"operator : GREATER",
"operator : LESS",
"operator : EQUAL",
"operator : NOTEQUAL",
"operator : CHANGED",
"time :",
"time : SECOND",
"time : MINUTE",
"time : HOUR",
"time : DAY",
"action : ALERT",
"action : EXEC argumentlist",
"action : EXEC argumentlist useroptionlist",
"action : RESTART",
"action : START",
"action : STOP",
"action : UNMONITOR",
"action1 : action",
"action2 : action",
"rate1 :",
"rate1 : NUMBER CYCLE",
"rate1 : NUMBER NUMBER CYCLE",
"rate2 :",
"rate2 : NUMBER CYCLE",
"rate2 : NUMBER NUMBER CYCLE",
"recovery :",
"recovery : ELSE IF RECOVERED rate2 THEN action2",
"recovery : ELSE IF PASSED rate2 THEN action2",
"recovery : ELSE IF SUCCEEDED rate2 THEN action2",
"checksum : IF FAILED hashtype CHECKSUM rate1 THEN action1 recovery",
"checksum : IF FAILED hashtype CHECKSUM EXPECT STRING rate1 THEN action1 recovery",
"checksum : IF CHANGED hashtype CHECKSUM rate1 THEN action1",
"hashtype :",
"hashtype : MD5HASH",
"hashtype : SHA1HASH",
"inode : IF INODE operator NUMBER rate1 THEN action1 recovery",
"inode : IF INODE operator NUMBER PERCENT rate1 THEN action1 recovery",
"space : IF SPACE operator value unit rate1 THEN action1 recovery",
"space : IF SPACE operator NUMBER PERCENT rate1 THEN action1 recovery",
"fsflag : IF CHANGED FSFLAG rate1 THEN action1",
"unit :",
"unit : BYTE",
"unit : KILOBYTE",
"unit : MEGABYTE",
"unit : GIGABYTE",
"permission : IF FAILED PERMISSION NUMBER rate1 THEN action1 recovery",
"match : IF matchflagnot MATCH PATH rate1 THEN action1",
"match : IF matchflagnot MATCH STRING rate1 THEN action1",
"match : IGNORE matchflagnot MATCH PATH",
"match : IGNORE matchflagnot MATCH STRING",
"matchflagnot :",
"matchflagnot : NOT",
"size : IF SIZE operator NUMBER unit rate1 THEN action1 recovery",
"size : IF CHANGED SIZE rate1 THEN action1",
"uid : IF FAILED UID STRING rate1 THEN action1 recovery",
"uid : IF FAILED UID NUMBER rate1 THEN action1 recovery",
"gid : IF FAILED GID STRING rate1 THEN action1 recovery",
"gid : IF FAILED GID NUMBER rate1 THEN action1 recovery",
"icmptype : TYPE ICMPECHO",
"reminder :",
"reminder : REMINDER NUMBER",
"reminder : REMINDER NUMBER CYCLE",
};
#endif
#if YYDEBUG
#include <stdio.h>
#endif

/* define the initial stack-sizes */
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH  YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 500
#define YYMAXDEPTH  500
#endif
#endif

#define YYINITSTACKSIZE 500

int      yydebug;
int      yynerrs;
int      yyerrflag;
int      yychar;
short   *yyssp;
YYSTYPE *yyvsp;
YYSTYPE  yyval;
YYSTYPE  yylval;

/* variables for the parser stack */
static short   *yyss;
static short   *yysslim;
static YYSTYPE *yyvs;
static int      yystacksize;
#line 1649 "p.y"


/* -------------------------------------------------------- Parser interface */


/**
 * Syntactic error routine
 *
 * This routine is automatically called by the lexer!
 */
void yyerror(const char *s, ...) {
  
  long len;
  va_list ap;
  char *msg = NULL;

  ASSERT(s);
  
  va_start(ap,s);
  msg = Util_formatString(s, ap, &len);
  va_end(ap);
  
  LogError("%s:%i: Error: %s '%s'\n", currentfile, lineno, msg, yytext);
  cfg_errflag++;
 
  FREE(msg);
  
}

/**
 * Syntactical warning routine
 */
void yywarning(const char *s, ...) {
  
  long len;
  va_list ap;
  char *msg = NULL;

  ASSERT(s);
  
  va_start(ap,s);
  msg = Util_formatString(s, ap, &len);
  va_end(ap);
  
  LogWarning("%s:%i: Warning: %s '%s'\n", currentfile, lineno, msg, yytext);
 
  FREE(msg);
  
}

/**
 * Argument error routine
 */
void yyerror2(const char *s, ...) {
  
  long len;
  va_list ap;
  char *msg = NULL;

  ASSERT(s);
  
  va_start(ap,s);
  msg = Util_formatString(s, ap, &len);
  va_end(ap);
  
  LogError("%s:%i: Error: %s '%s'\n", argcurrentfile, arglineno, msg, argyytext);
  cfg_errflag++;
 
  FREE(msg);
  
}

/**
 * Argument warning routine
 */
void yywarning2(const char *s, ...) {
  
  long len;
  va_list ap;
  char *msg = NULL;

  ASSERT(s);
  
  va_start(ap,s);
  msg = Util_formatString(s, ap, &len);
  va_end(ap);
  
  LogWarning("%s:%i: Warning: %s '%s'\n", argcurrentfile, arglineno, msg, argyytext);
 
  FREE(msg);
  
}

/*
 * The Parser hook - start parsing the control file
 * Returns TRUE if parsing succeeded, otherwise FALSE
 */
int parse(char *controlfile) {

  ASSERT(controlfile);

  servicelist = tail = current = NULL;

  /*
   * Secure check the monitrc file. The run control file must have the
   * same uid as the REAL uid of this process, it must have permissions
   * no greater than 700 and it must not be a symbolic link.
   */
  if (! File_checkStat(controlfile, "control file", S_IRUSR|S_IWUSR|S_IXUSR))
    return FALSE;

  if ((yyin = fopen(controlfile,"r")) == (FILE *)NULL) {
    LogError("%s: Error: cannot open the control file '%s' -- %s\n", prog, controlfile, STRERROR);
    return FALSE;
  }

  currentfile = xstrdup(controlfile);

  /* 
   * Creation of the global service list is synchronized  
   */
  LOCK(Run.mutex)
    preparse();
    yyparse();
    fclose(yyin);
    /* Add the default general system service if not specified explicitly */
    if (!hassystem) {
      check_name(Run.localhostname);
      createservice(TYPE_SYSTEM, xstrdup(Run.localhostname), xstrdup(""), check_system);
    }
    /* If defined - add the last service to the service list */
    if (current) {
      addservice(current);
      FREE(current);
    }
    postparse();
  END_LOCK;

  FREE(currentfile);

  if (argyytext != NULL)
    FREE(argyytext);

  return(cfg_errflag == 0);
}


/* ----------------------------------------------------------------- Private */


/**
 * Initialize objects used by the parser.
 */
static void preparse() {
  int i;
  char localhost[STRLEN];

  /*
   * Get the localhost name
   */
  if (gethostname(localhost, sizeof(localhost)) < 0)
    snprintf(localhost, STRLEN, "%s", LOCALHOST);

  /* Set instance incarnation ID */
  time(&Run.incarnation);
  /* Reset lexer */
  buffer_stack_ptr       = 0;
  lineno                 = 1;
  arglineno              = 1;
  argcurrentfile         = NULL;
  argyytext              = NULL;
  /* Reset parser */
  Run.stopped            = FALSE;
  Run.dolog              = FALSE;
  Run.dohttpd            = FALSE;
  Run.doaction           = FALSE;
  Run.httpdsig           = TRUE;
  Run.credentials        = NULL;
  Run.httpdssl           = FALSE;
  Run.httpsslpem         = NULL;
  Run.httpsslclientpem   = NULL;
  Run.clientssl          = FALSE;
  Run.mailserver_timeout = NET_TIMEOUT;
  Run.bind_addr          = NULL;
  Run.eventlist          = NULL;
  Run.eventlist_dir      = NULL;
  Run.eventlist_slots    = -1;
  Run.system             = NULL;
  Run.expectbuffer       = STRLEN;
  Run.mmonits            = NULL;
  Run.maillist           = NULL;
  Run.mailservers        = NULL;
  Run.MailFormat.from    = NULL;
  Run.MailFormat.subject = NULL;
  Run.MailFormat.message = NULL;
  Run.localhostname      = xstrdup(localhost);
  depend_list            = NULL;
  Run.handler_init       = TRUE;
  for (i = 0; i <= HANDLER_MAX; i++)
    Run.handler_queue[i] = 0;
  /* 
   * Initialize objects
   */
  reset_uidset();
  reset_gidset();
  reset_sizeset();
  reset_mailset();
  reset_mailserverset();
  reset_portset();
  reset_permset();
  reset_icmpset();
  reset_rateset();
  reset_filesystemset();
  reset_resourceset();
  reset_checksumset();
  reset_timestampset();
  reset_actionrateset();
}


/*
 * Check that values are reasonable after parsing
 */
static void postparse() {
  Service_T s;

  if (cfg_errflag || ! servicelist)
    return;

  /* Check the sanity of any dependency graph */
  check_depend();

  /* Check that we do not start monit in daemon mode without having a
   * poll time */
  if (!Run.polltime && (Run.isdaemon || Run.init)) {
    LogError("%s: Error: Poll time not defined. Please define poll time in the\n control file or use the -d option when starting monit\n", prog);
    cfg_errflag++;
  }

  if (Run.logfile)
    Run.dolog = TRUE;

  for (s = servicelist; s; s = s->next) {
    /* Set the general system service shortcut */
    if (s->type == TYPE_SYSTEM)
      Run.system = s;
    if (s->type != TYPE_HOST)
	continue;
    /* Verify that a remote service has a port or an icmp list */
    if (!s->portlist && !s->icmplist) {
      LogError("%s: Error: 'check host' statement is incomplete; Please specify a port number to test\n or an icmp test at the remote host: '%s'\n", prog, s->name);
      cfg_errflag++;
    }
  }
}


/*
 * Create a new service object and add any current objects to the
 * service list.
 */
static void createservice(int type, char *name, char *value, int (*check)(Service_T s)) {

  ASSERT(name);
  ASSERT(value);

  if (current)
    addservice(current);
  else
    NEW(current);

  /* Reset the current object */
  memset(current, 0, sizeof(*current));

  NEW(current->inf);
  Util_resetInfo(current);

  /* Set default values */
  current->monitor = MONITOR_INIT;
  current->mode    = MODE_ACTIVE;
  current->name    = name;
  current->type    = type;
  current->check   = check;
  current->path    = value;

  /* Initialize general event handlers */
  addeventaction(&(current)->action_DATA,     ACTION_ALERT,     ACTION_ALERT);
  addeventaction(&(current)->action_EXEC,     ACTION_ALERT,     ACTION_ALERT);
  addeventaction(&(current)->action_INVALID,  ACTION_RESTART,   ACTION_ALERT);
  addeventaction(&(current)->action_NONEXIST, ACTION_RESTART,   ACTION_ALERT);
  addeventaction(&(current)->action_PID,      ACTION_ALERT,     ACTION_IGNORE);
  addeventaction(&(current)->action_PPID,     ACTION_ALERT,     ACTION_IGNORE);
  addeventaction(&(current)->action_FSFLAG,   ACTION_ALERT,     ACTION_IGNORE);

  /* Initialize internal event handlers */
  addeventaction(&(current)->action_MONIT_START,  ACTION_START, ACTION_IGNORE);
  addeventaction(&(current)->action_MONIT_STOP,   ACTION_STOP,  ACTION_IGNORE);
  addeventaction(&(current)->action_MONIT_RELOAD, ACTION_START, ACTION_IGNORE);
  addeventaction(&(current)->action_ACTION,       ACTION_ALERT, ACTION_IGNORE);
  
  pthread_mutex_init(&current->mutex, NULL);

  gettimeofday(&current->collected, NULL);

}


/*
 * Add a service object to the servicelist
 */
static void addservice(Service_T s) {
  Service_T n;

  ASSERT(s);
 
  NEW(n);
  memcpy(n, s, sizeof(*s));
  /* Add the service to the end of the service list */
  if (tail != NULL) {
    tail->next = n;
    tail->next_conf = n;
  } else {
    servicelist = n;
    servicelist_conf = n;
  }
  tail = n;
}


/* 
 * Add entry to service group list
 */
static void addservicegroup(char *name) {
  ServiceGroup_T g;
  ServiceGroupMember_T m;

  ASSERT(name);
 
  /* Check if service group with the same name is defined already */
  for (g = servicegrouplist; g; g = g->next)
    if (! strcasecmp(g->name, name))
      break;

  if (! g) {
    NEW(g);
    g->name = xstrdup(name);
    g->next = servicegrouplist;
    servicegrouplist = g;
  }

  NEW(m);
  m->name = xstrdup(current->name);
  m->next = g->members;
  g->members = m;
}


/* 
 * Add a dependant entry to the current service dependant list
 *
 */
static void adddependant(char *dependant) {
  Dependant_T d; 

  ASSERT(dependant);
  
  NEW(d);
  
  if (current->dependantlist != NULL)
    d->next = current->dependantlist;

  d->dependant = dependant;
  current->dependantlist = d;

}


/*
 * Add the given mailaddress with the apropriate alert notification
 * values and mail attributes to the given mailinglist.
 */
static void addmail(char *mailto, Mail_T f, Mail_T *l, unsigned int events, unsigned int reminder) {
  Mail_T m;

  ASSERT(mailto);

  NEW(m);
  m->events   = events;
  m->to       = mailto;
  m->from     = f->from;
  m->subject  = f->subject;
  m->message  = f->message;
  m->reminder = reminder;
  
  m->next = *l;
  *l = m;

  reset_mailset();
}


/*
 * Add the given portset to the current service's portlist
 */
static void addport(Port_T port) {
  Port_T p;
  char address[STRLEN];
  
  ASSERT(port);

  NEW(p);
  p->port               = port->port;
  p->type               = port->type;
  p->socket             = port->socket;
  p->family             = port->family;
  p->action             = port->action;
  p->timeout            = port->timeout;
  p->request            = port->request;
  p->generic            = port->generic;
  p->protocol           = port->protocol;
  p->pathname           = port->pathname;
  p->hostname           = port->hostname;
  p->url_request        = port->url_request;
  p->request_checksum   = port->request_checksum;
  p->request_hostheader = port->request_hostheader;
  memcpy(&p->ApacheStatus, &port->ApacheStatus, sizeof(struct apache_status));

  if (p->request_checksum) {
    cleanup_hash_string(p->request_checksum);
    if (strlen(p->request_checksum) == 32)
      p->request_hashtype = HASH_MD5;
    else if (strlen(p->request_checksum) == 40)
      p->request_hashtype = HASH_SHA1;
    else
      yyerror2("invalid checksum [%s]", p->request_checksum);
  } else
    p->request_hashtype = 0;

  if (port->family == AF_INET)
    snprintf(address, STRLEN, "INET[%s:%d]", port->hostname, port->port);
  else if (port->family == AF_UNIX)
    snprintf(address, STRLEN, "UNIX[%s]", port->pathname);
  else
    address[0] = 0;
  p->address = xstrdup(address);
  
  if (port->SSL.use_ssl == TRUE) {
    if (!have_ssl()) {
      yyerror("ssl check cannot be activated. SSL is not supported");
    } else {
      if (port->SSL.certmd5 != NULL) {
	p->SSL.certmd5 = port->SSL.certmd5;
	cleanup_hash_string(p->SSL.certmd5);
      }
      p->SSL.use_ssl = TRUE;
      p->SSL.version = port->SSL.version;
    }
  }
  p->maxforward = port->maxforward;
  p->next = current->portlist;
  current->portlist = p;
  
  reset_portset();

}


/*
 * Add a new resource object to the current service resource list
 */
static void addresource(Resource_T rr) {
  Resource_T r;

  ASSERT(rr);

  NEW(r);
  if (! Run.doprocess)
    yyerror("Cannot activate service check. The process status engine was disabled. On certain systems you must run monit as root to utilize this feature)\n");
  r->resource_id = rr->resource_id;
  r->limit       = rr->limit;
  r->action      = rr->action;
  r->operator    = rr->operator;
  r->next        = current->resourcelist;

  current->resourcelist = r;
  reset_resourceset();
}


/*
 * Add a new file object to the current service timestamp list
 */
static void addtimestamp(Timestamp_T ts, int notime) {
  Timestamp_T t;

  ASSERT(ts);

  NEW(t);
  t->operator     = ts->operator;
  t->time         = ts->time;
  t->action       = ts->action;
  t->test_changes = ts->test_changes;
  
  if (t->test_changes || notime) {
    if (! File_exist(current->path)) {
      DEBUG("%s: Debug: the path '%s' used in the TIMESTAMP statement refer to a non-existing object\n", prog, current->path);
    } else if (!(t->timestamp = File_getTimestamp(current->path, S_IFDIR|S_IFREG))) {
      yyerror2("cannot get the timestamp for '%s'", current->path);
    }
  }
  
  t->next = current->timestamplist;
  current->timestamplist = t;

  reset_timestampset();
}


/*
 * Add a new object to the current service actionrate list
 */
static void addactionrate(ActionRate_T ar) {
  ActionRate_T a;

  ASSERT(ar);

  if (ar->count > ar->cycle)
    yyerror2("the number of restarts must be less than poll cycles");
  if (ar->count <= 0 || ar->cycle <= 0)
    yyerror2("zero or negative values not allowed in a action rate statement");

  NEW(a);
  a->count  = ar->count;
  a->cycle  = ar->cycle;
  a->action = ar->action;

  a->next = current->actionratelist;
  current->actionratelist = a;

  reset_actionrateset();
}



/*
 * Add a new Size object to the current service size list
 */
static void addsize(Size_T ss) {
  Size_T s;
  struct stat buf;

  ASSERT(ss);

  NEW(s);
  s->operator     = ss->operator;
  s->size         = ss->size;
  s->action       = ss->action;
  s->test_changes = ss->test_changes;
  /* Get the initial size for future comparision, if the file exists */
  if (s->test_changes) {
    s->test_changes_ok = !stat(current->path, &buf);
    if (s->test_changes_ok)
      s->size = (unsigned long long)buf.st_size;
  }
 
  s->next = current->sizelist;
  current->sizelist = s;

  reset_sizeset();
}


/*
 * Set Checksum object in the current service
 */
static void addchecksum(Checksum_T cs) {

  int len;
  Checksum_T c;

  ASSERT(cs);

  cs->test_changes_ok = TRUE;

  if (!cs->hash) {
    if (cs->type == HASH_UNKNOWN)
      cs->type = DEFAULT_HASH;
    if ( !(cs->hash = Util_getChecksum(current->path, cs->type))) {
      if (cs->test_changes == TRUE) {
        /* If the file doesn't exist and we're checking for checksum changes, set dummy value */
        cs->test_changes_ok = FALSE;
        cs->hash = xstrdup("00000000000000000000000000000000");
      } else {
        yyerror2("cannot compute a checksum for file %s", current->path);
        reset_checksumset();
        return;
      }
    }
  }

  len = cleanup_hash_string(cs->hash);

  if (cs->type == HASH_UNKNOWN) {
    if (len == 32) {
      cs->type = HASH_MD5;
    } else if (len == 40) {
      cs->type = HASH_SHA1;
    } else {
      yyerror2("invalid checksum [%s] for file %s", cs->hash, current->path);
      reset_checksumset();
      return;
    }
  } else if (( cs->type == HASH_MD5 && len!=32 ) || ( cs->type == HASH_SHA1 && len != 40 )) {
    yyerror2("invalid checksum [%s] for file %s", cs->hash, current->path);
    reset_checksumset();
    return;
  }

  NEW(c);

  c->type            = cs->type;
  c->hash            = cs->hash;
  c->test_changes    = cs->test_changes;
  c->test_changes_ok = cs->test_changes_ok;
  c->action          = cs->action;
 
  current->checksum = c;

  reset_checksumset();

}


/*
 * Set Perm object in the current service
 */
static void addperm(Perm_T ps) {
  Perm_T p;

  ASSERT(ps);

  NEW(p);
  p->perm       = ps->perm;
  p->action     = ps->action;
  current->perm = p;
  reset_permset();

}

/*
 * Set Match object in the current service
 */
static void addmatch(Match_T ms, int actionnumber, int linenumber) {
  Match_T m;
  Match_T ml;
  int     reg_return;
  
  ASSERT(ms);

  NEW(m);
#ifdef HAVE_REGEX_H
  NEW(m->regex_comp);
#endif

  m->match_string = ms->match_string;
  m->match_path   = ms->match_path ? xstrdup(ms->match_path) : NULL;
  m->action       = ms->action;
  m->not          = ms->not;
  m->ignore       = ms->ignore;
  m->next         = NULL;

  addeventaction(&(m->action), actionnumber, ACTION_IGNORE);

#ifdef HAVE_REGEX_H
  reg_return = regcomp(m->regex_comp, ms->match_string, REG_NOSUB|REG_EXTENDED);

  if (reg_return != 0) {
    char errbuf[STRLEN];
    regerror(reg_return, ms->regex_comp, errbuf, STRLEN);
    if (m->match_path != NULL) 
      yyerror2("regex parsing error:%s on line %i of", errbuf, linenumber);
    else
      yyerror2("regex parsing error:%s", errbuf);
  }
#endif

  if (current->matchlist) {
    /* Find the end of the list */
    for (ml = current->matchlist; ml->next; ml = ml->next)
      ;

    ml->next = m;
    
  } else
    current->matchlist = m;
}


static void addmatchpath(Match_T ms, int actionnumber) {

  FILE *handle;
  Command_T savecommand = NULL;
  char buf[2048];
  int linenumber = 0;

  ASSERT(ms->match_path);

  handle = fopen(ms->match_path, "r");
  if (handle == NULL) {
    yyerror2("cannot read regex match file (%s)", ms->match_path);
    return;
  }
  
  while (!feof(handle)) {
    int len;

    linenumber++;
    
    if (! fgets(buf, 2048, handle))
      continue;

    len = strlen(buf);

    if (len == 0 || buf[0] == '\n')
      continue;

    if (buf[len-1] == '\n')
      buf[len-1] = 0;

    ms->match_string = xstrdup(buf);

    /* The addeventaction() called from addmatch() will reset the
     * command1 to NULL, but we need to duplicate the command for
     * each line, thus need to save it here */
    if (actionnumber == ACTION_EXEC) {
      if (command1 == NULL) {
        ASSERT(savecommand);
        command1 = savecommand;
      }
      savecommand = copycommand(command1);
    }

    addmatch(ms, actionnumber, linenumber);
  }

  if (actionnumber == ACTION_EXEC && savecommand)
    gccmd(&savecommand);

  fclose(handle);
}


/*
 * Set Uid object in the current service
 */
static void adduid(Uid_T us) {
  Uid_T u;

  ASSERT(us);

  NEW(u);
  u->uid       = us->uid;
  u->action    = us->action;
  current->uid = u;
  reset_uidset();
}


/*
 * Set Gid object in the current service
 */
static void addgid(Gid_T gs) {
  Gid_T g;

  ASSERT(gs);

  NEW(g);
  g->gid       = gs->gid;
  g->action    = gs->action;
  current->gid = g;
  reset_gidset();
}


/*
 * Add a new filesystem to the current service's filesystem list
 */
static void addfilesystem(Filesystem_T ds) {
  Filesystem_T dev;

  ASSERT(ds);
  
  NEW(dev);
  dev->resource           = ds->resource;
  dev->operator           = ds->operator;
  dev->limit_absolute     = ds->limit_absolute;
  dev->limit_percent      = ds->limit_percent;
  dev->action             = ds->action;

  dev->next               = current->filesystemlist;
  current->filesystemlist = dev;

  reset_filesystemset();

}


/*
 * Add a new icmp object to the current service's icmp list
 */
static void addicmp(Icmp_T is) {
  if (!getuid()) {
    Icmp_T icmp;

    ASSERT(is);

    NEW(icmp);
    icmp->type         = is->type;      
    icmp->count        = is->count > ICMP_MAX_COUNT ? ICMP_MAX_COUNT : is->count;     
    icmp->timeout      = is->timeout;
    icmp->action       = is->action;
    icmp->is_available = FALSE;
    icmp->response     = -1;
  
    icmp->next         = current->icmplist;
    current->icmplist  = icmp;
  } else
    yyerror("icmp statements must be run as root");
  reset_icmpset();
}


/*
 * Set EventAction object
 */
static void addeventaction(EventAction_T *_ea, int failed, int succeeded) {
  EventAction_T ea;

  ASSERT(_ea);

  NEW(ea);
  NEW(ea->failed);
  NEW(ea->succeeded);

  ea->failed->id     = failed;
  ea->failed->count  = rate1.count;
  ea->failed->cycles = rate1.cycles;
  if (failed == ACTION_EXEC) {
    ASSERT(command1);
    ea->failed->exec = command1;
    command1 = NULL;
  }
  describeAction(ea->failed);

  ea->succeeded->id     = succeeded;
  ea->succeeded->count  = rate2.count;
  ea->succeeded->cycles = rate2.cycles;
  if (succeeded == ACTION_EXEC) {
    ASSERT(command2);
    ea->succeeded->exec = command2;
    command2 = NULL;
  }
  describeAction(ea->succeeded);
  *_ea = ea;
  reset_rateset();
}


/*
 * Redefine EventAction object (used for default action overloading)
 */
static void seteventaction(EventAction_T *_ea, int failed, int succeeded) {
  EventAction_T ea = *_ea;

  ASSERT(ea);
  ASSERT(ea->failed);
  ASSERT(ea->succeeded);

  ea->failed->id     = failed;
  ea->failed->count  = rate1.count;
  ea->failed->cycles = rate1.cycles;
  if (failed == ACTION_EXEC) {
    ASSERT(command1);
    ea->failed->exec = command1;
    command1 = NULL;
  }
  describeAction(ea->failed);

  ea->succeeded->id     = succeeded;
  ea->succeeded->count  = rate2.count;
  ea->succeeded->cycles = rate2.cycles;
  if (succeeded == ACTION_EXEC) {
    ASSERT(command2);
    ea->succeeded->exec = command2;
    command2 = NULL;
  }
  describeAction(ea->succeeded);
}


/*
 * Return a protocol object for the given protocol
 */
static void *addprotocol(int protocol) {

  switch (protocol) {
  case P_APACHESTATUS:  return create_apache_status();
  case P_DNS:           return create_dns();
  case P_DWP:           return create_dwp();
  case P_FTP:           return create_ftp();
  case P_GENERIC:       return create_generic();
  case P_HTTP:          return create_http();
  case P_IMAP:          return create_imap();
  case P_CLAMAV:        return create_clamav();
  case P_LDAP2:         return create_ldap2();
  case P_LDAP3:         return create_ldap3();
  case P_MYSQL:         return create_mysql();
  case P_NNTP:          return create_nntp();
  case P_NTP3:          return create_ntp3();
  case P_POSTFIXPOLICY: return create_postfix_policy();
  case P_POP:           return create_pop();
  case P_SMTP:          return create_smtp();
  case P_SSH:           return create_ssh();
  case P_RDATE:         return create_rdate();
  case P_RSYNC:         return create_rsync();
  case P_TNS:           return create_tns();
  case P_PGSQL:         return create_pgsql();
  case P_SIP:           return create_sip();
  case P_LMTP:          return create_lmtp();
  case P_GPS:           return create_gps();
  case P_RADIUS:        return create_radius();
  }

  return create_default();
   
}


/*
 * Add a generic protocol handler to 
 */
static void addgeneric(Port_T port, char *send, char *expect) {
  Generic_T g = port->generic;
  
  if (g == NULL) {
    NEW(g);
    port->generic = g;
  } else {
    while (g->next != NULL)
      g = g->next;
    NEW(g->next);
    g = g->next;
  }
  
  if (send != NULL) {
    g->send = xstrdup(send);
    g->expect = NULL;
  } else if (expect != NULL) {
#ifdef HAVE_REGEX_H
    
    int   reg_return;
    NEW(g->expect);
    reg_return = regcomp(g->expect, expect, REG_NOSUB|REG_EXTENDED);
    if (reg_return != 0) {
      char errbuf[STRLEN];
      regerror(reg_return, g->expect, errbuf, STRLEN);
      yyerror2("regex parsing error:%s", errbuf);
    }
#else
    g->expect = xstrdup(expect);
#endif
    g->send = NULL;
  } 
}


/*
 * Add the current command object to the current service object's
 * start or stop program.
 */
static void addcommand(int what, unsigned timeout) {

  switch(what) {
  case START: current->start = command; break;
  case STOP:  current->stop = command; break;
  }

  command->timeout = timeout;
  
  command = NULL;
  
}

  
/*
 * Add a new argument to the argument list
 */
static void addargument(char *argument) {

  ASSERT(argument);

  if (! command) {
    
    NEW(command);
    check_exec(argument);
    
  }
  
  command->arg[command->length++] = argument;
  command->arg[command->length] = NULL;
  
  if (command->length >= ARGMAX)
    yyerror("exceeded maximum number of program arguments");
  
}


/*
 * Setup a url request for the current port object
 */
static void prepare_urlrequest(URL_T U) {

  ASSERT(U);
  
  portset.protocol = addprotocol(P_HTTP);

  if (urlrequest == NULL)
    NEW(urlrequest);
  urlrequest->url = U;
  portset.hostname = xstrdup(U->hostname);
  check_hostname(portset.hostname);
  portset.port = U->port;
  portset.url_request = urlrequest;
  portset.type = SOCK_STREAM;
  portset.request = Util_getString("%s%s%s", U->path, U->query ? "?" : "", U->query ? U->query : "");
  /* Only the HTTP protocol is supported for URLs.
     See also the lexer if this is to be changed in
     the future */
  portset.protocol = addprotocol(P_HTTP);
  if (IS(U->protocol, "https"))
    portset.SSL.use_ssl = TRUE;
  
}


/*
 * Set the url request for a port
 */
static void  seturlrequest(int operator, char *regex) {
  
  ASSERT(regex);

  if (! urlrequest)
    NEW(urlrequest);
  urlrequest->operator = operator;
#ifdef HAVE_REGEX_H
  {    
    int reg_return;
    NEW(urlrequest->regex);
    reg_return = regcomp(urlrequest->regex, regex, REG_NOSUB|REG_EXTENDED);
    if (reg_return != 0) {
      char errbuf[STRLEN];
      regerror(reg_return, urlrequest->regex, errbuf, STRLEN);
      yyerror2("regex parsing error: %s", errbuf);
    }
  }
#else
  urlrequest->regex = xstrdup(regex);
#endif

}


/*
 * Add a new data recipient server to the mmonit server list
 */
static void addmmonit(URL_T url, int timeout, int sslversion, char *certmd5) {
  Mmonit_T c;
  
  ASSERT(url);

  NEW(c);
  c->url = url;
  if (!strcmp(c->url->protocol, "https")) {
    if (!have_ssl()) {
      yyerror("ssl check cannot be activated. SSL is not supported");
    } else {
      c->ssl.use_ssl = TRUE;
      c->ssl.version = (sslversion == SSL_VERSION_NONE) ? SSL_VERSION_AUTO : sslversion;
      if (certmd5) {
	c->ssl.certmd5 = certmd5;
	cleanup_hash_string(c->ssl.certmd5);
      }
    }
  }
  c->timeout = timeout;
  c->next = NULL;

  if (Run.mmonits) {
    Mmonit_T C;
    for (C = Run.mmonits; C->next; C = C->next)
      /* Empty */ ;
    C->next = c;
  } else {
    Run.mmonits = c;
  }
}


/*
 * Add a new smtp server to the mail server list
 */
static void addmailserver(MailServer_T mailserver) {

  MailServer_T s;
  
  ASSERT(mailserver->host);

  NEW(s);
  s->host        = mailserver->host;
  s->port        = mailserver->port;
  s->username    = mailserver->username;
  s->password    = mailserver->password;
  s->ssl.use_ssl = mailserver->ssl.use_ssl;
  s->ssl.version = mailserver->ssl.version;
  s->ssl.certmd5 = mailserver->ssl.certmd5;

  s->next = NULL;

  if (Run.mailservers) {
    MailServer_T l;
    for (l = Run.mailservers; l->next; l = l->next) /* empty */;
    l->next = s;
  } else {
    Run.mailservers = s;
  }
  
  reset_mailserverset();
}


/*
 * Return uid if found on the system. If the parameter user is NULL
 * the uid parameter is used for looking up the user id on the system,
 * otherwise the user parameter is used.
 */
static uid_t get_uid(char *user, uid_t uid) {
  struct passwd *pwd;

  if (user) {
    pwd = getpwnam(user);

    if (pwd == NULL) {
      yyerror2("requested user not found on the system");
      return(0);
    }

  } else {

    if ( (pwd = getpwuid(uid)) == NULL ) {
      yyerror2("requested uid not found on the system");
      return(0);
    }
  }

  return(pwd->pw_uid);

}


/*
 * Return gid if found on the system. If the parameter group is NULL
 * the gid parameter is used for looking up the group id on the system,
 * otherwise the group parameter is used.
 */
static gid_t get_gid(char *group, gid_t gid) {
  struct group *grd;

  if (group) {
    grd = getgrnam(group);

    if (grd == NULL) {
      yyerror2("requested group not found on the system");
      return(0);
    }

  } else {

    if ( (grd = getgrgid(gid)) == NULL ) {
      yyerror2("requested gid not found on the system");
      return(0);
    }

  }

  return(grd->gr_gid);

}


/*
 * Add a new user id to the current command object.
 */
static void addeuid(uid_t uid) {
  if (!getuid()) {
    command->has_uid = TRUE;
    command->uid = uid;
  } else
    yyerror("uid statement requires root privileges");
}


/*
 * Add a new group id to the current command object.
 */
static void addegid(gid_t gid) {
  if (!getuid()) {
    command->has_gid = TRUE;
    command->gid = gid;
  } else
    yyerror("gid statement requires root privileges");
}


/*
 * Reset the logfile if changed
 */
static void setlogfile(char *logfile) {
  if (Run.logfile) {
    if (IS(Run.logfile, logfile)) {
      FREE(logfile);
      return;
    } else
      FREE(Run.logfile);
  }
  Run.logfile = logfile;
}


/*
 * Reset the pidfile if changed
 */
static void setpidfile(char *pidfile) {
  if (Run.pidfile) {
    if (IS(Run.pidfile, pidfile)) {
      FREE(pidfile);
      return;
    } else
      FREE(Run.pidfile);
  }
  Run.pidfile = pidfile;
}


/*
 * Read a apache htpasswd file and add credentials found for username
 */
static void addhtpasswdentry(char *filename, char *username, int dtype) {
  char *ht_username = NULL;
  char *ht_passwd = NULL;
  char buf[STRLEN];
  FILE *handle = NULL;
  int credentials_added = 0;
  
  ASSERT(filename);

  handle = fopen(filename, "r");

  if ( handle == NULL ) {
    if (username != NULL)
      yyerror2("cannot read htpasswd (%s)", filename);
    else
      yyerror2("cannot read htpasswd", filename);
    return;
  }
  
  while (!feof(handle)) {
    char *colonindex = NULL;
    int i;
    
    if (! fgets(buf, STRLEN, handle))
      continue;

    /* strip trailing non visible characters */
    for (i = strlen(buf)-1; i >= 0; i--) {
      if ( buf[i] == ' '  || buf[i] == '\r' || buf[i] == '\n' || buf[i] == '\t' )
        buf[i] ='\0';
      else
        break;
    }

    if ( NULL == (colonindex = strchr(buf, ':')))
      continue;

    ht_passwd = xstrdup(colonindex+1);
    *colonindex = '\0';

    /* In case we have a file in /etc/passwd or /etc/shadow style we
     *  want to remove ":.*$" and Crypt and MD5 hashed dont have a colon
     */ 
    
    if ( (NULL != (colonindex = strchr(ht_passwd, ':'))) && ( dtype != DIGEST_CLEARTEXT) )
      *colonindex = '\0';

    ht_username = xstrdup(buf);

    if (username == NULL) {
      if (addcredentials(ht_username, ht_passwd, dtype, FALSE))
        credentials_added++;
    } else if (strcmp(username, ht_username) == 0)  {
      if (addcredentials(ht_username, ht_passwd, dtype, FALSE))
        credentials_added++;
    } else {
      FREE(ht_passwd);
      FREE(ht_username);
    }
  }

  if (credentials_added == 0) {
    if ( username == NULL )
      yywarning2("htpasswd file (%s) has no usable credentials", filename);
    else
      yywarning2("htpasswd file (%s) has no usable credentials for user %s", filename, username);
  }
  fclose(handle);
}


#ifdef HAVE_LIBPAM
static void addpamauth(char* groupname, int readonly) {
  Auth_T c = NULL;
  Auth_T prev = NULL;

  ASSERT(groupname);

  if (Run.credentials == NULL)
    NEW(Run.credentials);

  c = Run.credentials;
  do {
    if (c->groupname != NULL && IS(c->groupname, groupname)) {
      yywarning2("PAM group %s was added already, entry ignored", groupname);
      FREE(groupname);
      return;
    }
    prev = c;
    c = c->next;
  } while (c != NULL);

  NEW(prev->next);
  c = prev->next;

  c->next        = NULL;
  c->uname       = NULL;
  c->passwd      = NULL;
  c->groupname   = groupname;
  c->digesttype  = DIGEST_PAM;
  c->is_readonly = readonly;
  
  DEBUG("%s: Adding PAM group '%s'.\n", prog, groupname); 

  return;
}
#endif


/*
 * Add Basic Authentication credentials
 */
static int addcredentials(char *uname, char *passwd, int dtype, int readonly) {
  Auth_T c;

  ASSERT(uname);
  ASSERT(passwd);

  if (Run.credentials == NULL) {
    NEW(Run.credentials);
    c = Run.credentials;
  } else {

    if (Util_getUserCredentials(uname) != NULL) {
      yywarning2("credentials for user %s were already added, entry ignored", uname);
      FREE(uname);
      FREE(passwd);
      return FALSE;
    }

    c = Run.credentials;

    while ( c->next != NULL )
      c = c->next;

    NEW(c->next);
    c = c->next;
        
  }
  
  c->next        = NULL;
  c->uname       = uname;
  c->passwd      = passwd;
  c->groupname   = NULL;
  c->digesttype  = dtype;
  c->is_readonly = readonly;
  
  DEBUG("%s: Debug: Adding credentials for user '%s'.\n", prog, uname); 
  
  return TRUE;
  
}


/*
 * Set the syslog and the facilities to be used
 */
static void setsyslog(char *facility) {

  if (!Run.logfile || ihp.logfile) {
    ihp.logfile = TRUE;
    setlogfile(xstrdup("syslog"));
    Run.use_syslog = TRUE;
    Run.dolog = TRUE;
  }

  if (facility) {
    if (IS(facility,"log_local0"))
      Run.facility = LOG_LOCAL0;
    else if (IS(facility, "log_local1"))
      Run.facility = LOG_LOCAL1;
    else if (IS(facility, "log_local2"))
      Run.facility = LOG_LOCAL2;
    else if (IS(facility, "log_local3"))
      Run.facility = LOG_LOCAL3;
    else if (IS(facility, "log_local4"))
      Run.facility = LOG_LOCAL4;
    else if (IS(facility, "log_local5"))
      Run.facility = LOG_LOCAL5;
    else if (IS(facility, "log_local6"))
      Run.facility = LOG_LOCAL6;
    else if (IS(facility, "log_local7"))
      Run.facility = LOG_LOCAL7;
    else if (IS(facility, "log_daemon"))
      Run.facility = LOG_DAEMON;
    else
      yyerror2("invalid syslog facility");
  } else {
    Run.facility = LOG_USER;
  }
  
}


/*
 * Reset the current mailset, eventset and reminder for reuse
 */
static void reset_mailset() {
  memset(&mailset, 0, sizeof(struct mymail));
  eventset = EVENT_NULL;
}


/*
 * Reset the mailserver set to default values
 */
static void reset_mailserverset() {
  memset(&mailserverset, 0, sizeof(struct mymailserver));
  mailserverset.port = PORT_SMTP;
  mailserverset.ssl.use_ssl = FALSE;
  mailserverset.ssl.version = SSL_VERSION_AUTO;
}


/*
 * Reset the Port set to default values
 */
static void reset_portset() {
  memset(&portset, 0, sizeof(struct myport));
  portset.socket = -1;
  portset.type = SOCK_STREAM;
  portset.family = AF_INET;
  portset.SSL.version = SSL_VERSION_AUTO;
  portset.timeout = NET_TIMEOUT;
  portset.maxforward = 70;
  urlrequest = NULL;
}


/*
 * Reset the Proc set to default values
 */
static void reset_resourceset() {
  resourceset.resource_id = 0;
  resourceset.limit = 0;
  resourceset.action = NULL;
  resourceset.operator = OPERATOR_EQUAL;
}


/*
 * Reset the Timestamp set to default values
 */
static void reset_timestampset() {
  timestampset.operator = OPERATOR_EQUAL;
  timestampset.time = 0;
  timestampset.test_changes = FALSE;
  timestampset.action = NULL;
}


/*
 * Reset the ActionRate set to default values
 */
static void reset_actionrateset() {
  actionrateset.count = 0;
  actionrateset.cycle = 0;
  actionrateset.action = NULL;
}


/*
 * Reset the Size set to default values
 */
static void reset_sizeset() {
  sizeset.operator = OPERATOR_EQUAL;
  sizeset.size = 0;
  sizeset.test_changes = FALSE;
  sizeset.action = NULL;
}


/*
 * Reset the Checksum set to default values
 */
static void reset_checksumset() {
  checksumset.type = HASH_UNKNOWN;
  checksumset.hash = NULL;
  checksumset.test_changes = FALSE;
  checksumset.action = NULL;
}


/*
 * Reset the Perm set to default values
 */
static void reset_permset() {
  permset.perm = 0;
  permset.action = NULL;
}


/*
 * Reset the Uid set to default values
 */
static void reset_uidset() {
  uidset.uid = 0;
  uidset.action = NULL;
}


/*
 * Reset the Gid set to default values
 */
static void reset_gidset() {
  gidset.gid = 0;
  gidset.action = NULL;
}


/*
 * Reset the Filesystem set to default values
 */
static void reset_filesystemset() {
  filesystemset.resource = 0;
  filesystemset.operator = OPERATOR_EQUAL;
  filesystemset.limit_absolute = -1;
  filesystemset.limit_percent = -1;
  filesystemset.action = NULL;
}


/*
 * Reset the ICMP set to default values
 */
static void reset_icmpset() {
  icmpset.type = ICMP_ECHO;
  icmpset.count = ICMP_ATTEMPT_COUNT;
  icmpset.timeout = NET_TIMEOUT;
  icmpset.action = NULL;
}


/*
 * Reset the Rate set to default values
 */
static void reset_rateset() {
  rate1.count  = 1;
  rate1.cycles = 1;

  rate2.count  = 1;
  rate2.cycles = 1;
}


/* ---------------------------------------------------------------- Checkers */


/*
 * Check for unique service name
 */
static void check_name(char *name) {

  ASSERT(name);

  if (Util_existService(name) || (current && IS(name, current->name)))
    yyerror2("service name conflict, %s already defined", name);
  if (name && *name == '/')
    yyerror2("service name must not start with '/' -- ", name);
  
}


/*
 * Permission statement semantic check
 */
static int check_perm(int perm) {
  long result;
  char *status;
  char buf[STRLEN];

  snprintf(buf, STRLEN, "%d", perm);

  result = strtol(buf, &status, 8);

  if ( *status != '\0' || result < 0 || result > 07777 )
    yyerror2("permission statements must have an octal value between 0 and 7777");

  return result;
}


/*
 * Every statement semantic check
 */
static void check_every(int every) {
  if (every <= 1)
    yyerror2("an EVERY statement must have a value greater than 1");
}


/*
 * Check hostname 
 */
static void check_hostname(char *hostname) {

  ASSERT(hostname);

  if (!check_host(hostname))
    yywarning2("hostname did not resolve");
}

/*
 * Check the dependency graph for errors
 * by doing a topological sort, thereby finding any cycles.
 * Assures that graph is a Directed Acyclic Graph (DAG).
 */
static void check_depend() {
  Service_T s;
  Service_T depends_on = NULL;
  Service_T* dlt = &depend_list; /* the current tail of it                                 */
  int done;                      /* no unvisited nodes left?                               */
  int found_some;                /* last iteration found anything new ?                    */
  depend_list = NULL;            /* depend_list will be the topological sorted servicelist */

  do { 
    done = TRUE;
    found_some = FALSE; 
    for (s = servicelist; s; s = s->next) {
        Dependant_T d;
      if (s->visited)
	  continue;
      done = FALSE; // still unvisited nodes
      depends_on = NULL;
      for (d = s->dependantlist; d; d = d->next) {
        Service_T dp = Util_getService(d->dependant);
        if (!dp) {
          LogError("%s: Error: Depend service '%s' is not defined in the control file\n", prog, d->dependant);
          exit(1);
        }
        if (!dp->visited) {
          depends_on = dp;
        }
      }

      if (!depends_on) {
        s->visited = TRUE;
        found_some = TRUE;
        *dlt = s;
        dlt = &s->next_depend;
      }
    }
  } while (found_some && !done);
	
  if (!done) {
        ASSERT(depends_on);
	LogError("%s: Error: Found a depend loop in the control file involving the service '%s'\n", prog, depends_on->name);
	exit(1);
   } 

  ASSERT(depend_list);
  servicelist = depend_list;
    
  for (s = depend_list; s; s = s->next_depend)
    s->next = s->next_depend;
    
  reset_depend();
}


/*
 * Check if the executable exist
 */
static void check_exec(char *exec) {
  if (! File_exist(exec))
    yywarning2("the executable does not exist");
}
 
 
/* Return a valid max forward value for SIP header */
static int verifyMaxForward(int mf) { 
  int max = 70;
  
  if (mf >= 0 && mf <= 255)
    max = mf;
  else
    yywarning2("SIP max forward is outside the range [0..255]. Setting max forward to 70");
  
  return max;
}


/* -------------------------------------------------------------------- Misc */


/*
 * Cleans up an md5 string, tolower and remove byte separators
 */
static int cleanup_hash_string(char *hashstring) {
  int i = 0, j = 0;

  ASSERT(hashstring);

  while (hashstring[i] != '\0') {
    if (isxdigit((int) hashstring[i])) {
      hashstring[j] = tolower((int)hashstring[i]);
      j++;
    } 
    i++;
  }
  hashstring[j] = '\0';
  return j;
}


static void describeAction(Action_T A) {
  #define BUF_CURSOR    (A->description + strlen(A->description))
  #define BUF_AVAILABLE (sizeof(A->description) - strlen(A->description))
  snprintf(BUF_CURSOR, BUF_AVAILABLE, "%s", actionnames[A->id]);
  if (A->id == ACTION_EXEC) {
    int i = 0;
    Command_T C = A->exec;

    while (C->arg[i]) {
      snprintf(BUF_CURSOR, BUF_AVAILABLE, "%s%s", i ? " " : " '", C->arg[i]);
      i++;
    }
    snprintf(BUF_CURSOR, BUF_AVAILABLE, "'");
    if (C->has_uid)
      snprintf(BUF_CURSOR, BUF_AVAILABLE, " as uid %d", C->uid);
    if (C->has_gid)
      snprintf(BUF_CURSOR, BUF_AVAILABLE, " as gid %d", C->gid);
    snprintf(BUF_CURSOR, BUF_AVAILABLE, " timeout %d cycle(s)", C->timeout);
  }
  #undef BUF_CURSOR
  #undef BUF_AVAILABLE
}


/* Return deep copy of the command */
static Command_T copycommand(Command_T source) {
  int i;
  Command_T copy = NULL;

  NEW(copy);
  copy->length = source->length;
  copy->has_uid = source->has_uid;
  copy->uid = source->uid;
  copy->has_gid = source->has_gid;
  copy->gid = source->gid;
  copy->timeout = source->timeout;
  for (i = 0; i < copy->length; i++)
     copy->arg[i] = xstrdup(source->arg[i]);
  copy->arg[copy->length] = NULL;

  return copy;
}

#line 3574 "y.tab.c"
/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack(void)
{
    int newsize, i;
    short *newss;
    YYSTYPE *newvs;

    if ((newsize = yystacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return -1;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;

    i = yyssp - yyss;
    newss = (yyss != 0)
          ? (short *)realloc(yyss, newsize * sizeof(*newss))
          : (short *)malloc(newsize * sizeof(*newss));
    if (newss == 0)
        return -1;

    yyss  = newss;
    yyssp = newss + i;
    newvs = (yyvs != 0)
          ? (YYSTYPE *)realloc(yyvs, newsize * sizeof(*newvs))
          : (YYSTYPE *)malloc(newsize * sizeof(*newvs));
    if (newvs == 0)
        return -1;

    yyvs = newvs;
    yyvsp = newvs + i;
    yystacksize = newsize;
    yysslim = yyss + newsize - 1;
    return 0;
}

#define YYABORT goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR goto yyerrlab
int
yyparse(void)
{
    register int yym, yyn, yystate;
#if YYDEBUG
    register const char *yys;

    if ((yys = getenv("YYDEBUG")) != 0)
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yynerrs = 0;
    yyerrflag = 0;
    yychar = YYEMPTY;

    if (yyss == NULL && yygrowstack()) goto yyoverflow;
    yyssp = yyss;
    yyvsp = yyvs;
    *yyssp = yystate = 0;

yyloop:
    if ((yyn = yydefred[yystate]) != 0) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yyssp >= yysslim && yygrowstack())
        {
            goto yyoverflow;
        }
        *++yyssp = yystate = yytable[yyn];
        *++yyvsp = yylval;
        yychar = YYEMPTY;
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;

    yyerror("syntax error");

#ifdef lint
    goto yyerrlab;
#endif

yyerrlab:
    ++yynerrs;

yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yyssp]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yyssp, yytable[yyn]);
#endif
                if (yyssp >= yysslim && yygrowstack())
                {
                    goto yyoverflow;
                }
                *++yyssp = yystate = yytable[yyn];
                *++yyvsp = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yyssp);
#endif
                if (yyssp <= yyss) goto yyabort;
                --yyssp;
                --yyvsp;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = YYEMPTY;
        goto yyloop;
    }

yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    if (yym)
        yyval = yyvsp[1-yym];
    else
        memset(&yyval, 0, sizeof yyval);
    switch (yyn)
    {
case 129:
#line 476 "p.y"
{
                    addmail(yyvsp[-5].string, &mailset, &Run.maillist, eventset, yyvsp[0].number);
                  }
break;
case 130:
#line 479 "p.y"
{
                    addmail(yyvsp[-2].string, &mailset, &Run.maillist, EVENT_ALL, yyvsp[0].number);
                  }
break;
case 131:
#line 482 "p.y"
{
                   addmail(yyvsp[-6].string, &mailset, &Run.maillist, ~eventset, yyvsp[0].number);
                  }
break;
case 132:
#line 487 "p.y"
{
                    if (!Run.isdaemon || ihp.daemon) {
                      ihp.daemon     = TRUE;
                      Run.isdaemon   = TRUE;
                      Run.polltime   = yyvsp[-1].number;
                      Run.startdelay = yyvsp[0].number;
                    }
                  }
break;
case 133:
#line 497 "p.y"
{ yyval.number = START_DELAY; }
break;
case 134:
#line 498 "p.y"
{ yyval.number = yyvsp[0].number; }
break;
case 135:
#line 501 "p.y"
{
                    Run.expectbuffer = yyvsp[-1].number * yyvsp[0].number;
                  }
break;
case 136:
#line 506 "p.y"
{
                    Run.init = TRUE;
                  }
break;
case 137:
#line 511 "p.y"
{
                   if (!Run.logfile || ihp.logfile) {
                     ihp.logfile = TRUE;
                     setlogfile(yyvsp[0].string);
                     Run.use_syslog = FALSE;
                     Run.dolog =TRUE;
                   }
                  }
break;
case 138:
#line 519 "p.y"
{
                    setsyslog(NULL);
                  }
break;
case 139:
#line 522 "p.y"
{
                    setsyslog(yyvsp[0].string); FREE(yyvsp[0].string);
                  }
break;
case 140:
#line 527 "p.y"
{
                    Run.eventlist_dir = yyvsp[0].string;
                  }
break;
case 141:
#line 530 "p.y"
{
                    Run.eventlist_dir = yyvsp[-2].string;
                    Run.eventlist_slots = yyvsp[0].number;
                  }
break;
case 142:
#line 534 "p.y"
{
                    Run.eventlist_dir = xstrdup(MYEVENTLISTBASE);
                    Run.eventlist_slots = yyvsp[0].number;
                  }
break;
case 143:
#line 540 "p.y"
{
                    Run.idfile = yyvsp[0].string;
                  }
break;
case 144:
#line 545 "p.y"
{
                    Run.statefile = yyvsp[0].string;
                  }
break;
case 145:
#line 550 "p.y"
{
                   if (!Run.pidfile || ihp.pidfile) {
                     ihp.pidfile = TRUE;
                     setpidfile(yyvsp[0].string);
                   }
                 }
break;
case 149:
#line 565 "p.y"
{
                    check_hostname((yyvsp[-3].url)->hostname);
                    addmmonit(yyvsp[-3].url, yyvsp[-2].number, yyvsp[-1].number, yyvsp[0].string); 
                  }
break;
case 150:
#line 571 "p.y"
{
                   Run.mailserver_timeout = yyvsp[-1].number;
                   Run.mail_hostname = yyvsp[0].string;
                  }
break;
case 151:
#line 577 "p.y"
{
                   Run.MailFormat.from    = mailset.from    ?  mailset.from    : xstrdup(ALERT_FROM);
                   Run.MailFormat.subject = mailset.subject ?  mailset.subject : xstrdup(ALERT_SUBJECT);
                   Run.MailFormat.message = mailset.message ?  mailset.message : xstrdup(ALERT_MESSAGE);
                   reset_mailset();
                 }
break;
case 152:
#line 585 "p.y"
{
                   Run.dohttpd = TRUE;
                   Run.httpdport = yyvsp[-1].number;
                 }
break;
case 155:
#line 595 "p.y"
{
                    /* Restore the current text overriden by lookahead */
                    FREE(argyytext);
                    argyytext = xstrdup(yyvsp[-4].string);

                    check_hostname(yyvsp[-4].string);
                    mailserverset.host = yyvsp[-4].string;
                    mailserverset.username = yyvsp[-3].string;
                    mailserverset.password = yyvsp[-2].string;
                    mailserverset.ssl.version = yyvsp[-1].number;
                    if (mailserverset.ssl.version != SSL_VERSION_NONE) {
                      mailserverset.ssl.use_ssl = TRUE;
                      if (mailserverset.ssl.version == SSL_VERSION_SSLV2 ||
                         mailserverset.ssl.version == SSL_VERSION_SSLV3)
                         mailserverset.port = PORT_SMTPS;
                      mailserverset.ssl.certmd5 = yyvsp[0].string;
                    }
                    addmailserver(&mailserverset);
                  }
break;
case 156:
#line 614 "p.y"
{
                    /* Restore the current text overriden by lookahead */
                    FREE(argyytext);
                    argyytext = xstrdup(yyvsp[-6].string);

                    check_hostname(yyvsp[-6].string);
                    mailserverset.host = yyvsp[-6].string;
                    mailserverset.port = yyvsp[-4].number;
                    mailserverset.username = yyvsp[-3].string;
                    mailserverset.password = yyvsp[-2].string;
                    mailserverset.ssl.version = yyvsp[-1].number;
                    if (mailserverset.ssl.version != SSL_VERSION_NONE) {
                      mailserverset.ssl.use_ssl = TRUE;
                      mailserverset.ssl.certmd5 = yyvsp[0].string;
                    }
                    addmailserver(&mailserverset);
                  }
break;
case 163:
#line 643 "p.y"
{ Run.httpdssl = FALSE; }
break;
case 164:
#line 644 "p.y"
{ 
                    Run.httpdssl = TRUE;                   
                    if (!have_ssl())
                      yyerror("SSL is not supported");
                  }
break;
case 170:
#line 658 "p.y"
{ FREE(yyvsp[0].string); }
break;
case 171:
#line 659 "p.y"
{ FREE(yyvsp[0].string); }
break;
case 173:
#line 663 "p.y"
{ Run.httpdsig = TRUE; }
break;
case 174:
#line 664 "p.y"
{ Run.httpdsig = FALSE; }
break;
case 179:
#line 675 "p.y"
{ Run.bind_addr = yyvsp[0].string; }
break;
case 180:
#line 678 "p.y"
{
                    Run.httpsslpem = yyvsp[0].string;
                    if (!File_checkStat(Run.httpsslpem, "SSL server PEM file", S_IRWXU))
                      yyerror2("SSL server PEM file has too loose permissions");
                  }
break;
case 182:
#line 686 "p.y"
{
                    Run.httpsslclientpem = yyvsp[0].string; 
                    Run.clientssl = TRUE;
                    if (!File_checkStat(Run.httpsslclientpem, "SSL client PEM file", S_IRWXU | S_IRGRP | S_IROTH))
                      yyerror2("SSL client PEM file has too loose permissions");
                  }
break;
case 183:
#line 694 "p.y"
{ 
                   Run.allowselfcert = FALSE; 
                   config_ssl(Run.allowselfcert); 
                 }
break;
case 184:
#line 698 "p.y"
{   
                    Run.allowselfcert = TRUE;
                    config_ssl(Run.allowselfcert); 
                  }
break;
case 185:
#line 704 "p.y"
{
                    addcredentials(yyvsp[-3].string,yyvsp[-1].string, DIGEST_CLEARTEXT, yyvsp[0].number);
                  }
break;
case 186:
#line 707 "p.y"
{
#ifdef HAVE_LIBPAM
                    addpamauth(yyvsp[-1].string, yyvsp[0].number);
#else
                    yyerror("PAM is not supported");
                    FREE(yyvsp[-1].string);
#endif
                  }
break;
case 187:
#line 715 "p.y"
{
                    addhtpasswdentry(yyvsp[0].string, NULL, DIGEST_CLEARTEXT);
                    FREE(yyvsp[0].string);
                  }
break;
case 188:
#line 719 "p.y"
{
                    addhtpasswdentry(yyvsp[0].string, NULL, DIGEST_CLEARTEXT);
                    FREE(yyvsp[0].string);
                  }
break;
case 189:
#line 723 "p.y"
{
                    addhtpasswdentry(yyvsp[0].string, NULL, DIGEST_MD5);
                    FREE(yyvsp[0].string);
                  }
break;
case 190:
#line 727 "p.y"
{
                    addhtpasswdentry(yyvsp[0].string, NULL, DIGEST_CRYPT);
                    FREE(yyvsp[0].string);
                  }
break;
case 191:
#line 731 "p.y"
{
                    htpasswd_file = yyvsp[0].string;
                    digesttype = CLEARTEXT;
                  }
break;
case 192:
#line 735 "p.y"
{
                    FREE(htpasswd_file);
                  }
break;
case 193:
#line 738 "p.y"
{
                    htpasswd_file = yyvsp[0].string;
                    digesttype = DIGEST_CLEARTEXT;
                  }
break;
case 194:
#line 742 "p.y"
{
                    FREE(htpasswd_file);
                  }
break;
case 195:
#line 745 "p.y"
{
                    htpasswd_file = yyvsp[0].string;
                    digesttype = DIGEST_MD5;
                  }
break;
case 196:
#line 749 "p.y"
{
                    FREE(htpasswd_file);
                  }
break;
case 197:
#line 752 "p.y"
{
                    htpasswd_file = yyvsp[0].string;
                    digesttype = DIGEST_CRYPT;
                  }
break;
case 198:
#line 756 "p.y"
{
                    FREE(htpasswd_file);
                  }
break;
case 199:
#line 759 "p.y"
{
                    if (! (add_net_allow(yyvsp[0].string) || add_host_allow(yyvsp[0].string))) {
                      yyerror2("erroneous network or host identifier %s", yyvsp[0].string); 
                    }
                    FREE(yyvsp[0].string);
                  }
break;
case 202:
#line 771 "p.y"
{ addhtpasswdentry(htpasswd_file, yyvsp[0].string, digesttype);
                           FREE(yyvsp[0].string); }
break;
case 203:
#line 775 "p.y"
{ yyval.number = FALSE; }
break;
case 204:
#line 776 "p.y"
{ yyval.number = TRUE; }
break;
case 205:
#line 779 "p.y"
{
                    check_name(yyvsp[-2].string);
                    createservice(TYPE_PROCESS, yyvsp[-2].string, yyvsp[0].string, check_process);
                  }
break;
case 206:
#line 783 "p.y"
{
                    check_name(yyvsp[-2].string);
                    createservice(TYPE_PROCESS, yyvsp[-2].string, yyvsp[0].string, check_process);
                  }
break;
case 207:
#line 789 "p.y"
{
                    check_name(yyvsp[-2].string); 
                    createservice(TYPE_FILE, yyvsp[-2].string, yyvsp[0].string, check_file);
                  }
break;
case 208:
#line 795 "p.y"
{
                    check_name(yyvsp[-2].string);
                    createservice(TYPE_FILESYSTEM, yyvsp[-2].string, yyvsp[0].string, check_filesystem);
                  }
break;
case 209:
#line 801 "p.y"
{
                    check_name(yyvsp[-2].string);
                    createservice(TYPE_DIRECTORY, yyvsp[-2].string, yyvsp[0].string, check_directory);
                  }
break;
case 210:
#line 807 "p.y"
{
                    check_hostname(yyvsp[0].string); 
                    check_name(yyvsp[-2].string);
                    createservice(TYPE_HOST, yyvsp[-2].string, yyvsp[0].string, check_remote_host);
                  }
break;
case 211:
#line 814 "p.y"
{
                    check_name(yyvsp[0].string);
                    createservice(TYPE_SYSTEM, yyvsp[0].string, xstrdup(""), check_system);
                    hassystem = TRUE;
                  }
break;
case 212:
#line 821 "p.y"
{
                    check_name(yyvsp[-2].string);
                    createservice(TYPE_FIFO, yyvsp[-2].string, yyvsp[0].string, check_fifo);
                  }
break;
case 213:
#line 827 "p.y"
{
                    check_name(yyvsp[-2].string);
                    createservice(TYPE_STATUS, yyvsp[-2].string, yyvsp[0].string, check_status);
                  }
break;
case 214:
#line 833 "p.y"
{
                    addcommand(START, yyvsp[0].number);
                  }
break;
case 215:
#line 836 "p.y"
{
                    addcommand(START, yyvsp[0].number);
                  }
break;
case 216:
#line 841 "p.y"
{
                    addcommand(STOP, yyvsp[0].number);
                  }
break;
case 217:
#line 844 "p.y"
{
                    addcommand(STOP, yyvsp[0].number);
                  }
break;
case 222:
#line 857 "p.y"
{ addargument(yyvsp[0].string); }
break;
case 223:
#line 858 "p.y"
{ addargument(yyvsp[0].string); }
break;
case 224:
#line 861 "p.y"
{ addeuid( get_uid(yyvsp[0].string, 0) ); FREE(yyvsp[0].string); }
break;
case 225:
#line 862 "p.y"
{ addegid( get_gid(yyvsp[0].string, 0) ); FREE(yyvsp[0].string); }
break;
case 226:
#line 863 "p.y"
{ addeuid( get_uid(NULL, yyvsp[0].number) ); }
break;
case 227:
#line 864 "p.y"
{ addegid( get_gid(NULL, yyvsp[0].number) ); }
break;
case 228:
#line 867 "p.y"
{ yyval.string = NULL; }
break;
case 229:
#line 868 "p.y"
{ yyval.string = yyvsp[0].string; }
break;
case 230:
#line 869 "p.y"
{ yyval.string = yyvsp[0].string; }
break;
case 231:
#line 872 "p.y"
{ yyval.string = NULL; }
break;
case 232:
#line 873 "p.y"
{ yyval.string = yyvsp[0].string; }
break;
case 233:
#line 876 "p.y"
{ yyval.string = NULL; }
break;
case 234:
#line 877 "p.y"
{ yyval.string = yyvsp[0].string; }
break;
case 235:
#line 881 "p.y"
{
                    portset.timeout = yyvsp[-4].number;
                    addeventaction(&(portset).action, yyvsp[-1].number, yyvsp[0].number);
                    addport(&portset);
                  }
break;
case 236:
#line 887 "p.y"
{ 
                    prepare_urlrequest(yyvsp[-6].url);
                    portset.timeout = yyvsp[-4].number;
                    addeventaction(&(portset).action, yyvsp[-1].number, yyvsp[0].number);
                    addport(&portset);
                  }
break;
case 237:
#line 896 "p.y"
{
                   portset.timeout = yyvsp[-4].number;
                   addeventaction(&(portset).action, yyvsp[-1].number, yyvsp[0].number);
                   addport(&portset);
                  }
break;
case 238:
#line 904 "p.y"
{
                   icmpset.type = yyvsp[-6].number;
                   icmpset.count = yyvsp[-5].number;
                   icmpset.timeout = yyvsp[-4].number;
                   addeventaction(&(icmpset).action, yyvsp[-1].number, yyvsp[0].number);
                   addicmp(&icmpset);
                  }
break;
case 239:
#line 913 "p.y"
{
                    if (current->type == TYPE_HOST)
                      portset.hostname = xstrdup(current->path);
                    else
                      portset.hostname = xstrdup(LOCALHOST);
                  }
break;
case 240:
#line 919 "p.y"
{ check_hostname(yyvsp[0].string); portset.hostname = yyvsp[0].string; }
break;
case 241:
#line 922 "p.y"
{ portset.port = yyvsp[0].number; portset.family = AF_INET; }
break;
case 242:
#line 925 "p.y"
{
                    portset.pathname = yyvsp[0].string; portset.family = AF_UNIX;
                  }
break;
case 243:
#line 930 "p.y"
{
                    portset.type = SOCK_STREAM;
                  }
break;
case 244:
#line 933 "p.y"
{
                    portset.type = SOCK_STREAM;
                  }
break;
case 245:
#line 936 "p.y"
{
                    portset.type = SOCK_STREAM;
                    portset.SSL.use_ssl = TRUE;
                    portset.SSL.version = yyvsp[-1].number;
                    if (portset.SSL.version == SSL_VERSION_NONE)
                      portset.SSL.version = SSL_VERSION_AUTO;
                    portset.SSL.certmd5 = yyvsp[0].string;
                  }
break;
case 246:
#line 944 "p.y"
{
                    portset.type = SOCK_DGRAM;
                  }
break;
case 247:
#line 949 "p.y"
{ yyval.string = NULL; }
break;
case 248:
#line 950 "p.y"
{ yyval.string = yyvsp[0].string; }
break;
case 249:
#line 953 "p.y"
{ yyval.number = SSL_VERSION_NONE; }
break;
case 250:
#line 954 "p.y"
{ yyval.number = SSL_VERSION_SSLV2; }
break;
case 251:
#line 955 "p.y"
{ yyval.number = SSL_VERSION_SSLV3; }
break;
case 252:
#line 956 "p.y"
{ yyval.number = SSL_VERSION_TLS; }
break;
case 253:
#line 957 "p.y"
{ yyval.number = SSL_VERSION_AUTO; }
break;
case 254:
#line 960 "p.y"
{
                    portset.protocol = addprotocol(P_DEFAULT);
                  }
break;
case 255:
#line 963 "p.y"
{
                    portset.protocol = addprotocol(P_APACHESTATUS);
                  }
break;
case 256:
#line 966 "p.y"
{
                    portset.protocol = addprotocol(P_DEFAULT);
                  }
break;
case 257:
#line 969 "p.y"
{
                    portset.protocol = addprotocol(P_DNS);
                  }
break;
case 258:
#line 972 "p.y"
{
                    portset.protocol = addprotocol(P_DWP);
                  }
break;
case 259:
#line 975 "p.y"
{
                    portset.protocol = addprotocol(P_FTP);
                  }
break;
case 260:
#line 978 "p.y"
{
                    portset.protocol = addprotocol(P_HTTP);
                  }
break;
case 261:
#line 981 "p.y"
{
                    portset.protocol = addprotocol(P_IMAP);
                  }
break;
case 262:
#line 984 "p.y"
{
                    portset.protocol = addprotocol(P_CLAMAV);
                  }
break;
case 263:
#line 987 "p.y"
{
                    portset.protocol = addprotocol(P_LDAP2);
                  }
break;
case 264:
#line 990 "p.y"
{
                    portset.protocol = addprotocol(P_LDAP3);
                  }
break;
case 265:
#line 993 "p.y"
{
                    portset.protocol = addprotocol(P_MYSQL);
                  }
break;
case 266:
#line 996 "p.y"
{
                    portset.protocol = addprotocol(P_SIP);
                  }
break;
case 267:
#line 999 "p.y"
{
                    portset.protocol = addprotocol(P_NNTP);
                  }
break;
case 268:
#line 1002 "p.y"
{
                    portset.protocol = addprotocol(P_NTP3);
                    portset.type = SOCK_DGRAM;
                  }
break;
case 269:
#line 1006 "p.y"
{
                    portset.protocol = addprotocol(P_POSTFIXPOLICY);
                  }
break;
case 270:
#line 1009 "p.y"
{
                    portset.protocol = addprotocol(P_POP);
                  }
break;
case 271:
#line 1012 "p.y"
{
                    portset.protocol = addprotocol(P_SMTP);
                  }
break;
case 272:
#line 1015 "p.y"
{
                    portset.protocol = addprotocol(P_SSH);
                  }
break;
case 273:
#line 1018 "p.y"
{
                    portset.protocol = addprotocol(P_RDATE);
                  }
break;
case 274:
#line 1021 "p.y"
{
                    portset.protocol = addprotocol(P_RSYNC);
                  }
break;
case 275:
#line 1024 "p.y"
{
                    portset.protocol = addprotocol(P_TNS);
                  }
break;
case 276:
#line 1027 "p.y"
{
                    portset.protocol = addprotocol(P_PGSQL);
                  }
break;
case 277:
#line 1030 "p.y"
{
                    portset.protocol = addprotocol(P_LMTP);
                  }
break;
case 278:
#line 1033 "p.y"
{
                    portset.protocol = addprotocol(P_GPS);
                  }
break;
case 279:
#line 1036 "p.y"
{
                    portset.protocol = addprotocol(P_RADIUS);
                  }
break;
case 280:
#line 1039 "p.y"
{
                    portset.protocol = addprotocol(P_GENERIC);
                  }
break;
case 283:
#line 1048 "p.y"
{ addgeneric(&portset, yyvsp[0].string, NULL); FREE(yyvsp[0].string);}
break;
case 284:
#line 1049 "p.y"
{ addgeneric(&portset, NULL, yyvsp[0].string); FREE(yyvsp[0].string);}
break;
case 286:
#line 1053 "p.y"
{
                    portset.request = yyvsp[0].string;
                  }
break;
case 287:
#line 1056 "p.y"
{
                    portset.request = yyvsp[0].string;
                  }
break;
case 289:
#line 1062 "p.y"
{
                     portset.maxforward = verifyMaxForward(yyvsp[0].number); 
                   }
break;
case 291:
#line 1068 "p.y"
{ 
                    portset.request = Util_urlEncode(yyvsp[-1].string); 
                    FREE(yyvsp[-1].string); 
                  }
break;
case 292:
#line 1072 "p.y"
{
                    portset.request = Util_urlEncode(yyvsp[-3].string);
                    FREE(yyvsp[-3].string);
                    portset.request_checksum = yyvsp[-1].string;
                  }
break;
case 294:
#line 1080 "p.y"
{
                    portset.request_hostheader = yyvsp[0].string;
                  }
break;
case 295:
#line 1085 "p.y"
{ 
                    portset.request = yyvsp[0].string; 
                  }
break;
case 298:
#line 1094 "p.y"
{ 
                    portset.ApacheStatus.loglimitOP = yyvsp[-2].number; 
                    portset.ApacheStatus.loglimit = (int)yyvsp[-1].number; 
                  }
break;
case 299:
#line 1098 "p.y"
{ 
                    portset.ApacheStatus.closelimitOP = yyvsp[-2].number; 
                    portset.ApacheStatus.closelimit = (int)(yyvsp[-1].number); 
                  }
break;
case 300:
#line 1102 "p.y"
{ 
                    portset.ApacheStatus.dnslimitOP = yyvsp[-2].number; 
                    portset.ApacheStatus.dnslimit = (int)(yyvsp[-1].number); 
                  }
break;
case 301:
#line 1106 "p.y"
{ 
                    portset.ApacheStatus.keepalivelimitOP = yyvsp[-2].number; 
                    portset.ApacheStatus.keepalivelimit = (int)(yyvsp[-1].number); 
                  }
break;
case 302:
#line 1110 "p.y"
{ 
                    portset.ApacheStatus.replylimitOP = yyvsp[-2].number; 
                    portset.ApacheStatus.replylimit = (int)(yyvsp[-1].number); 
                  }
break;
case 303:
#line 1114 "p.y"
{ 
                    portset.ApacheStatus.requestlimitOP = yyvsp[-2].number; 
                    portset.ApacheStatus.requestlimit = (int)(yyvsp[-1].number); 
                  }
break;
case 304:
#line 1118 "p.y"
{ 
                    portset.ApacheStatus.startlimitOP = yyvsp[-2].number; 
                    portset.ApacheStatus.startlimit = (int)(yyvsp[-1].number); 
                  }
break;
case 305:
#line 1122 "p.y"
{ 
                    portset.ApacheStatus.waitlimitOP = yyvsp[-2].number; 
                    portset.ApacheStatus.waitlimit = (int)(yyvsp[-1].number); 
                  }
break;
case 306:
#line 1126 "p.y"
{ 
                    portset.ApacheStatus.gracefullimitOP = yyvsp[-2].number; 
                    portset.ApacheStatus.gracefullimit = (int)(yyvsp[-1].number); 
                  }
break;
case 307:
#line 1130 "p.y"
{ 
                    portset.ApacheStatus.cleanuplimitOP = yyvsp[-2].number; 
                    portset.ApacheStatus.cleanuplimit = (int)(yyvsp[-1].number); 
                  }
break;
case 308:
#line 1136 "p.y"
{
                    seteventaction(&(current)->action_PID, yyvsp[0].number, ACTION_IGNORE);
                  }
break;
case 309:
#line 1141 "p.y"
{
                    seteventaction(&(current)->action_PPID, yyvsp[0].number, ACTION_IGNORE);
                  }
break;
case 310:
#line 1146 "p.y"
{
                   yyval.number = ICMP_ATTEMPT_COUNT;
                  }
break;
case 311:
#line 1149 "p.y"
{    
                   yyval.number = yyvsp[0].number;
                  }
break;
case 312:
#line 1154 "p.y"
{
                   yyval.number = EXEC_TIMEOUT;
                  }
break;
case 313:
#line 1157 "p.y"
{
                   yyval.number = yyvsp[-1].number;
                  }
break;
case 314:
#line 1162 "p.y"
{
                   yyval.number = NET_TIMEOUT;
                  }
break;
case 315:
#line 1165 "p.y"
{
                   yyval.number = yyvsp[-1].number;
                  }
break;
case 316:
#line 1170 "p.y"
{
                   actionrateset.count = yyvsp[-5].number;
                   actionrateset.cycle = yyvsp[-3].number;
                   addeventaction(&(actionrateset).action, yyvsp[0].number, ACTION_IGNORE);
                   addactionrate(&actionrateset);
                 }
break;
case 317:
#line 1176 "p.y"
{
                   actionrateset.count = yyvsp[-5].number;
                   actionrateset.cycle = yyvsp[-3].number;
                   addeventaction(&(actionrateset).action, ACTION_UNMONITOR, ACTION_IGNORE);
                   addactionrate(&actionrateset);
                 }
break;
case 319:
#line 1185 "p.y"
{
                    seturlrequest(yyvsp[-1].number, yyvsp[0].string);
                    FREE(yyvsp[0].string);
                  }
break;
case 320:
#line 1191 "p.y"
{ yyval.number = OPERATOR_EQUAL; }
break;
case 321:
#line 1192 "p.y"
{ yyval.number = OPERATOR_NOTEQUAL; }
break;
case 322:
#line 1195 "p.y"
{
                   addmail(yyvsp[-5].string, &mailset, &current->maillist, eventset, yyvsp[0].number);
                  }
break;
case 323:
#line 1198 "p.y"
{
                   addmail(yyvsp[-2].string, &mailset, &current->maillist, EVENT_ALL, yyvsp[0].number);
                  }
break;
case 324:
#line 1201 "p.y"
{
                   addmail(yyvsp[-6].string, &mailset, &current->maillist, ~eventset, yyvsp[0].number);
                  }
break;
case 325:
#line 1204 "p.y"
{
                   addmail(yyvsp[0].string, &mailset, &current->maillist, EVENT_NULL, 0);
                  }
break;
case 326:
#line 1209 "p.y"
{ yyval.string = yyvsp[0].string; }
break;
case 327:
#line 1212 "p.y"
{ yyval.string = yyvsp[0].string; }
break;
case 330:
#line 1219 "p.y"
{ eventset |= EVENT_ACTION; }
break;
case 331:
#line 1220 "p.y"
{ eventset |= EVENT_CHECKSUM; }
break;
case 332:
#line 1221 "p.y"
{ eventset |= EVENT_CONNECTION; }
break;
case 333:
#line 1222 "p.y"
{ eventset |= EVENT_CONTENT; }
break;
case 334:
#line 1223 "p.y"
{ eventset |= EVENT_DATA; }
break;
case 335:
#line 1224 "p.y"
{ eventset |= EVENT_EXEC; }
break;
case 336:
#line 1225 "p.y"
{ eventset |= EVENT_FSFLAG; }
break;
case 337:
#line 1226 "p.y"
{ eventset |= EVENT_GID; }
break;
case 338:
#line 1227 "p.y"
{ eventset |= EVENT_ICMP; }
break;
case 339:
#line 1228 "p.y"
{ eventset |= EVENT_INSTANCE; }
break;
case 340:
#line 1229 "p.y"
{ eventset |= EVENT_INVALID; }
break;
case 341:
#line 1230 "p.y"
{ eventset |= EVENT_NONEXIST; }
break;
case 342:
#line 1231 "p.y"
{ eventset |= EVENT_PERMISSION; }
break;
case 343:
#line 1232 "p.y"
{ eventset |= EVENT_PID; }
break;
case 344:
#line 1233 "p.y"
{ eventset |= EVENT_PPID; }
break;
case 345:
#line 1234 "p.y"
{ eventset |= EVENT_RESOURCE; }
break;
case 346:
#line 1235 "p.y"
{ eventset |= EVENT_SIZE; }
break;
case 347:
#line 1236 "p.y"
{ eventset |= EVENT_TIMEOUT; }
break;
case 348:
#line 1237 "p.y"
{ eventset |= EVENT_TIMESTAMP; }
break;
case 349:
#line 1238 "p.y"
{ eventset |= EVENT_UID; }
break;
case 354:
#line 1249 "p.y"
{ mailset.from = yyvsp[0].string; }
break;
case 355:
#line 1250 "p.y"
{ mailset.subject = yyvsp[0].string; }
break;
case 356:
#line 1251 "p.y"
{ mailset.message = yyvsp[0].string; }
break;
case 357:
#line 1254 "p.y"
{
                   check_every(yyvsp[-1].number);
                   current->def_every = TRUE;
                   current->every = yyvsp[-1].number;
                 }
break;
case 358:
#line 1261 "p.y"
{
                    current->mode = MODE_ACTIVE;
                  }
break;
case 359:
#line 1264 "p.y"
{
                    current->mode = MODE_PASSIVE;
                  }
break;
case 360:
#line 1267 "p.y"
{
                    current->mode = MODE_MANUAL;
                    current->monitor = MONITOR_NOT;
                  }
break;
case 361:
#line 1273 "p.y"
{ addservicegroup(yyvsp[0].string); FREE(yyvsp[0].string);}
break;
case 365:
#line 1284 "p.y"
{ adddependant(yyvsp[0].string); }
break;
case 366:
#line 1287 "p.y"
{
                     addeventaction(&(resourceset).action, yyvsp[-1].number, yyvsp[0].number);
                     addresource(&resourceset);
                   }
break;
case 373:
#line 1303 "p.y"
{
                     addeventaction(&(resourceset).action, yyvsp[-1].number, yyvsp[0].number);
                     addresource(&resourceset);
                   }
break;
case 379:
#line 1318 "p.y"
{
                    resourceset.resource_id = RESOURCE_ID_CPU_PERCENT;
                    resourceset.operator = yyvsp[-2].number;
                    resourceset.limit = (yyvsp[-1].number * 10); 
                  }
break;
case 380:
#line 1323 "p.y"
{
                    resourceset.resource_id = RESOURCE_ID_TOTAL_CPU_PERCENT;
                    resourceset.operator = yyvsp[-2].number;
                    resourceset.limit = (yyvsp[-1].number * 10); 
                  }
break;
case 381:
#line 1330 "p.y"
{
                    resourceset.resource_id = yyvsp[-3].number;
                    resourceset.operator = yyvsp[-2].number;
                    resourceset.limit = (yyvsp[-1].number * 10); 
                  }
break;
case 382:
#line 1337 "p.y"
{ yyval.number = RESOURCE_ID_CPUUSER; }
break;
case 383:
#line 1338 "p.y"
{ yyval.number = RESOURCE_ID_CPUSYSTEM; }
break;
case 384:
#line 1339 "p.y"
{ yyval.number = RESOURCE_ID_CPUWAIT; }
break;
case 385:
#line 1342 "p.y"
{
                    resourceset.resource_id = RESOURCE_ID_MEM_KBYTE;
                    resourceset.operator = yyvsp[-2].number;
                    resourceset.limit = (int) (yyvsp[-1].real * (yyvsp[0].number / 1024.0)); 
                  }
break;
case 386:
#line 1347 "p.y"
{
                    resourceset.resource_id = RESOURCE_ID_MEM_PERCENT;
                    resourceset.operator = yyvsp[-2].number;
                    resourceset.limit = (yyvsp[-1].number * 10); 
                  }
break;
case 387:
#line 1352 "p.y"
{
                    resourceset.resource_id = RESOURCE_ID_TOTAL_MEM_KBYTE;
                    resourceset.operator = yyvsp[-2].number;
                    resourceset.limit = (int) (yyvsp[-1].real * (yyvsp[0].number / 1024.0));
                  }
break;
case 388:
#line 1357 "p.y"
{
                    resourceset.resource_id = RESOURCE_ID_TOTAL_MEM_PERCENT;
                    resourceset.operator = yyvsp[-2].number;
                    resourceset.limit = (yyvsp[-1].number * 10);
                  }
break;
case 389:
#line 1364 "p.y"
{ 
                    resourceset.resource_id = RESOURCE_ID_CHILDREN;
                    resourceset.operator = yyvsp[-1].number;
                    resourceset.limit = (int) yyvsp[0].number; 
                  }
break;
case 390:
#line 1371 "p.y"
{ 
                    resourceset.resource_id = yyvsp[-2].number;
                    resourceset.operator = yyvsp[-1].number;
                    resourceset.limit = (int) (yyvsp[0].real * 10.0); 
                  }
break;
case 391:
#line 1378 "p.y"
{ yyval.number = RESOURCE_ID_LOAD1; }
break;
case 392:
#line 1379 "p.y"
{ yyval.number = RESOURCE_ID_LOAD5; }
break;
case 393:
#line 1380 "p.y"
{ yyval.number = RESOURCE_ID_LOAD15; }
break;
case 394:
#line 1383 "p.y"
{ yyval.real = yyvsp[0].real; }
break;
case 395:
#line 1384 "p.y"
{ yyval.real = (float) yyvsp[0].number; }
break;
case 396:
#line 1387 "p.y"
{
                    timestampset.operator = yyvsp[-6].number;
                    timestampset.time = (yyvsp[-5].number * yyvsp[-4].number);
                    addeventaction(&(timestampset).action, yyvsp[-1].number, yyvsp[0].number);
                    addtimestamp(&timestampset, FALSE);
                  }
break;
case 397:
#line 1393 "p.y"
{
                    timestampset.test_changes = TRUE;
                    addeventaction(&(timestampset).action, yyvsp[0].number, ACTION_IGNORE);
                    addtimestamp(&timestampset, TRUE);
                  }
break;
case 398:
#line 1400 "p.y"
{ yyval.number = OPERATOR_EQUAL; }
break;
case 399:
#line 1401 "p.y"
{ yyval.number = OPERATOR_GREATER; }
break;
case 400:
#line 1402 "p.y"
{ yyval.number = OPERATOR_LESS; }
break;
case 401:
#line 1403 "p.y"
{ yyval.number = OPERATOR_EQUAL; }
break;
case 402:
#line 1404 "p.y"
{ yyval.number = OPERATOR_NOTEQUAL; }
break;
case 403:
#line 1405 "p.y"
{ yyval.number = OPERATOR_NOTEQUAL; }
break;
case 404:
#line 1408 "p.y"
{ yyval.number = TIME_SECOND; }
break;
case 405:
#line 1409 "p.y"
{ yyval.number = TIME_SECOND; }
break;
case 406:
#line 1410 "p.y"
{ yyval.number = TIME_MINUTE; }
break;
case 407:
#line 1411 "p.y"
{ yyval.number = TIME_HOUR; }
break;
case 408:
#line 1412 "p.y"
{ yyval.number = TIME_DAY; }
break;
case 409:
#line 1415 "p.y"
{ yyval.number = ACTION_ALERT; }
break;
case 410:
#line 1416 "p.y"
{ yyval.number = ACTION_EXEC; }
break;
case 411:
#line 1417 "p.y"
{ yyval.number = ACTION_EXEC; }
break;
case 412:
#line 1418 "p.y"
{ yyval.number = ACTION_RESTART; }
break;
case 413:
#line 1419 "p.y"
{ yyval.number = ACTION_START; }
break;
case 414:
#line 1420 "p.y"
{ yyval.number = ACTION_STOP; }
break;
case 415:
#line 1421 "p.y"
{ yyval.number = ACTION_UNMONITOR; }
break;
case 416:
#line 1424 "p.y"
{
                    yyval.number = yyvsp[0].number;
                    if (yyvsp[0].number == ACTION_EXEC && command) {
                      command1 = command;
                      command = NULL;
                    }
                  }
break;
case 417:
#line 1433 "p.y"
{
                    yyval.number = yyvsp[0].number;
                    if (yyvsp[0].number == ACTION_EXEC && command) {
                      command2 = command;
                      command = NULL;
                    }
                  }
break;
case 419:
#line 1443 "p.y"
{
                    rate1.count  = yyvsp[-1].number;
                    rate1.cycles = yyvsp[-1].number;
                    if (rate1.cycles < 1 || rate1.cycles > BITMAP_MAX)
                      yyerror2("the number of cycles must be between 1 and %d", BITMAP_MAX);
                  }
break;
case 420:
#line 1449 "p.y"
{
                    rate1.count  = yyvsp[-2].number;
                    rate1.cycles = yyvsp[-1].number;
                    if (rate1.cycles < 1 || rate1.cycles > BITMAP_MAX)
                      yyerror2("the number of cycles must be between 1 and %d", BITMAP_MAX);
                    if (rate1.count < 1 || rate1.count > rate1.cycles)
                      yyerror2("the number of events must be bigger then 0 and less than poll cycles");
                  }
break;
case 422:
#line 1460 "p.y"
{
                    rate2.count  = yyvsp[-1].number;
                    rate2.cycles = yyvsp[-1].number;
                    if (rate2.cycles < 1 || rate2.cycles > BITMAP_MAX)
                      yyerror2("the number of cycles must be between 1 and %d", BITMAP_MAX);
                  }
break;
case 423:
#line 1466 "p.y"
{
                    rate2.count  = yyvsp[-2].number;
                    rate2.cycles = yyvsp[-1].number;
                    if (rate2.cycles < 1 || rate2.cycles > BITMAP_MAX)
                      yyerror2("the number of cycles must be between 1 and %d", BITMAP_MAX);
                    if (rate2.count < 1 || rate2.count > rate2.cycles)
                      yyerror2("the number of events must be bigger then 0 and less than poll cycles");
                  }
break;
case 424:
#line 1476 "p.y"
{
                    yyval.number = ACTION_ALERT;
                  }
break;
case 425:
#line 1479 "p.y"
{
                    yyval.number = yyvsp[0].number;
                  }
break;
case 426:
#line 1482 "p.y"
{
                    yyval.number = yyvsp[0].number;
                  }
break;
case 427:
#line 1485 "p.y"
{
                    yyval.number = yyvsp[0].number;
                  }
break;
case 428:
#line 1490 "p.y"
{
                    addeventaction(&(checksumset).action, yyvsp[-1].number, yyvsp[0].number);
                    addchecksum(&checksumset);
                  }
break;
case 429:
#line 1495 "p.y"
{
                    checksumset.hash = yyvsp[-4].string;
                    addeventaction(&(checksumset).action, yyvsp[-1].number, yyvsp[0].number);
                    addchecksum(&checksumset);
                  }
break;
case 430:
#line 1500 "p.y"
{
                    checksumset.test_changes = TRUE;
                    addeventaction(&(checksumset).action, yyvsp[0].number, ACTION_IGNORE);
                    addchecksum(&checksumset);
                  }
break;
case 431:
#line 1506 "p.y"
{ checksumset.type = HASH_UNKNOWN; }
break;
case 432:
#line 1507 "p.y"
{ checksumset.type = HASH_MD5; }
break;
case 433:
#line 1508 "p.y"
{ checksumset.type = HASH_SHA1; }
break;
case 434:
#line 1511 "p.y"
{
                    filesystemset.resource = RESOURCE_ID_INODE;
                    filesystemset.operator = yyvsp[-5].number;
                    filesystemset.limit_absolute = yyvsp[-4].number;
                    addeventaction(&(filesystemset).action, yyvsp[-1].number, yyvsp[0].number);
                    addfilesystem(&filesystemset);
                  }
break;
case 435:
#line 1518 "p.y"
{
                    filesystemset.resource = RESOURCE_ID_INODE;
                    filesystemset.operator = yyvsp[-6].number;
                    filesystemset.limit_percent = (int)(yyvsp[-5].number * 10);
                    addeventaction(&(filesystemset).action, yyvsp[-1].number, yyvsp[0].number);
                    addfilesystem(&filesystemset);
                  }
break;
case 436:
#line 1527 "p.y"
{
                    if (!filesystem_usage(current->inf, current->path))
                      yyerror2("cannot read usage of filesystem %s", current->path);
                    filesystemset.resource = RESOURCE_ID_SPACE;
                    filesystemset.operator = yyvsp[-6].number;
                    filesystemset.limit_absolute = (int)((float)yyvsp[-5].real / (float)current->inf->f_bsize * (float)yyvsp[-4].number);
                    addeventaction(&(filesystemset).action, yyvsp[-1].number, yyvsp[0].number);
                    addfilesystem(&filesystemset);
                  }
break;
case 437:
#line 1536 "p.y"
{
                    filesystemset.resource = RESOURCE_ID_SPACE;
                    filesystemset.operator = yyvsp[-6].number;
                    filesystemset.limit_percent = (int)(yyvsp[-5].number * 10);
                    addeventaction(&(filesystemset).action, yyvsp[-1].number, yyvsp[0].number);
                    addfilesystem(&filesystemset);
                  }
break;
case 438:
#line 1545 "p.y"
{
                    seteventaction(&(current)->action_FSFLAG, yyvsp[0].number, ACTION_IGNORE);
                  }
break;
case 439:
#line 1550 "p.y"
{ yyval.number = UNIT_BYTE; }
break;
case 440:
#line 1551 "p.y"
{ yyval.number = UNIT_BYTE; }
break;
case 441:
#line 1552 "p.y"
{ yyval.number = UNIT_KILOBYTE; }
break;
case 442:
#line 1553 "p.y"
{ yyval.number = UNIT_MEGABYTE; }
break;
case 443:
#line 1554 "p.y"
{ yyval.number = UNIT_GIGABYTE; }
break;
case 444:
#line 1557 "p.y"
{
                    permset.perm = check_perm(yyvsp[-4].number);
                    addeventaction(&(permset).action, yyvsp[-1].number, yyvsp[0].number);
                    addperm(&permset);
                  }
break;
case 445:
#line 1564 "p.y"
{
                    matchset.ignore = FALSE;
                    matchset.match_path = yyvsp[-3].string;
                    matchset.match_string = NULL;
                    addmatchpath(&matchset, yyvsp[0].number);
                    FREE(yyvsp[-3].string); 
                  }
break;
case 446:
#line 1571 "p.y"
{
                    matchset.ignore = FALSE;
                    matchset.match_path = NULL;
                    matchset.match_string = xstrdup(yyvsp[-3].string);
                    addmatch(&matchset, yyvsp[0].number, 0);
                  }
break;
case 447:
#line 1577 "p.y"
{
                    matchset.ignore = TRUE;
                    matchset.match_path = yyvsp[0].string;
                    matchset.match_string = NULL;
                    addmatchpath(&matchset, ACTION_IGNORE);
                    FREE(yyvsp[0].string); 
                  }
break;
case 448:
#line 1584 "p.y"
{
                    matchset.ignore = TRUE;
                    matchset.match_path = NULL;
                    matchset.match_string = xstrdup(yyvsp[0].string);
                    addmatch(&matchset, ACTION_IGNORE, 0);
                  }
break;
case 449:
#line 1592 "p.y"
{
                    matchset.not = FALSE;
                  }
break;
case 450:
#line 1595 "p.y"
{
                    matchset.not = TRUE;
                  }
break;
case 451:
#line 1601 "p.y"
{
                    sizeset.operator = yyvsp[-6].number;
                    sizeset.size = ((unsigned long long)yyvsp[-5].number * yyvsp[-4].number);
                    addeventaction(&(sizeset).action, yyvsp[-1].number, yyvsp[0].number);
                    addsize(&sizeset);
                  }
break;
case 452:
#line 1607 "p.y"
{
                    sizeset.test_changes = TRUE;
                    addeventaction(&(sizeset).action, yyvsp[0].number, ACTION_IGNORE);
                    addsize(&sizeset);
                  }
break;
case 453:
#line 1614 "p.y"
{
                    uidset.uid = get_uid(yyvsp[-4].string, 0);
                    addeventaction(&(uidset).action, yyvsp[-1].number, yyvsp[0].number);
                    adduid(&uidset);
                    FREE(yyvsp[-4].string);
                  }
break;
case 454:
#line 1620 "p.y"
{
                    uidset.uid = get_uid(NULL, yyvsp[-4].number);
                    addeventaction(&(uidset).action, yyvsp[-1].number, yyvsp[0].number);
                    adduid(&uidset);
                  }
break;
case 455:
#line 1627 "p.y"
{
                    gidset.gid = get_gid(yyvsp[-4].string, 0);
                    addeventaction(&(gidset).action, yyvsp[-1].number, yyvsp[0].number);
                    addgid(&gidset);
                    FREE(yyvsp[-4].string);
                  }
break;
case 456:
#line 1633 "p.y"
{
                    gidset.gid = get_gid(NULL, yyvsp[-4].number);
                    addeventaction(&(gidset).action, yyvsp[-1].number, yyvsp[0].number);
                    addgid(&gidset);
                  }
break;
case 457:
#line 1640 "p.y"
{ yyval.number = ICMP_ECHO; }
break;
case 458:
#line 1643 "p.y"
{ yyval.number = 0; }
break;
case 459:
#line 1644 "p.y"
{ yyval.number = yyvsp[0].number; }
break;
case 460:
#line 1645 "p.y"
{ yyval.number = yyvsp[-1].number; }
break;
#line 5423 "y.tab.c"
    }
    yyssp -= yym;
    yystate = *yyssp;
    yyvsp -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yyssp = YYFINAL;
        *++yyvsp = yyval;
        if (yychar < 0)
        {
            if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yyssp, yystate);
#endif
    if (yyssp >= yysslim && yygrowstack())
    {
        goto yyoverflow;
    }
    *++yyssp = yystate;
    *++yyvsp = yyval;
    goto yyloop;

yyoverflow:
    yyerror("yacc stack overflow");

yyabort:
    return (1);

yyaccept:
    return (0);
}
